@@ -1,4 +1,4 @@
-; Centaur Miscellaneous Books
+; Centaur AIG Library
 ; Copyright (C) 2012 Centaur Technology
 ;
 ; Contact:
@@ -20,8 +20,6 @@
 ; Original author: Sol Swords <sswords@centtech.com>
 
 (in-package "ACL2")
-
-
 (include-book "tools/bstar" :dir :system)
 (include-book "tools/mv-nth" :dir :system)
 (include-book "data-structures/list-defthms" :dir :system)
@@ -31,11 +29,12 @@
 (include-book "centaur/misc/numlist" :dir :system)
 (include-book "tools/defmacfun" :dir :system)
 (include-book "misc/definline" :dir :system)
-(include-book "system/io" :dir :system)
+(include-book "std/io/base" :dir :system)
 (include-book "arithmetic/nat-listp" :dir :system)
 (local (include-book "clause-processors/instantiate" :dir :system))
 (local (include-book "centaur/bitops/ihsext-basics" :dir :system))
 (local (include-book "arithmetic/top-with-meta" :dir :system))
+(local (include-book "std/alists/top" :dir :system))
 (include-book "cutil/defmvtypes" :dir :system)
 
 (local (in-theory (disable reverse-removal revappend-removal)))
@@ -219,9 +218,11 @@
                   (mv-let
                     (names maxvar outs1 gates)
                     (aig-list-to-aiger aigs names maxvar gates)
-                    (mv names maxvar (revappend outs outs1) gates)))))
+                    (mv names maxvar (revappend outs outs1) gates))))
+  :hints(("Goal" :in-theory (enable revappend))))
 
-(verify-guards aig-list-to-aiger)
+(verify-guards aig-list-to-aiger
+  :hints(("Goal" :in-theory (enable revappend))))
 
 (defthm names-max-entry-of-aig-list-to-aiger
   (implies (and (natp maxvar)
@@ -381,7 +382,8 @@ OUTS, and GATES.
 (defthm aiger-gate-listp-revappend
   (implies (and (aiger-gate-listp x)
                 (aiger-gate-listp y))
-           (aiger-gate-listp (revappend x y))))
+           (aiger-gate-listp (revappend x y)))
+  :hints(("Goal" :in-theory (enable revappend))))
 
 (defthm aiger-gate-listp-append
   (implies (and (aiger-gate-listp x)
@@ -548,9 +550,11 @@ OUTS, and GATES.
 (defthm aiger-get-lits-tr-rw
   (implies (not (stringp acc))
            (equal (aiger-get-lits-tr list map acc)
-                  (revappend acc (aiger-get-lits list map)))))
+                  (revappend acc (aiger-get-lits list map))))
+  :hints(("Goal" :in-theory (enable revappend))))
 
-(verify-guards aiger-get-lits)
+(verify-guards aiger-get-lits
+  :hints(("Goal" :in-theory (enable revappend))))
 
 
 
@@ -1462,7 +1466,8 @@ CNSTR-AIGS - list of constraint AIGs. "
 
 (defthm nat-listp-revappend
   (implies (and (nat-listp a) (nat-listp b))
-           (nat-listp (revappend a b))))
+           (nat-listp (revappend a b)))
+  :hints(("Goal" :in-theory (enable revappend))))
 
 (defthm nat-listp-of-read-n-str-separated-nats-tr
   (implies (nat-listp acc)
