@@ -31,6 +31,7 @@
 (include-book "../gl/gify-clause-proc")
 (local (include-book "../gl/general-object-thms"))
 (local (include-book "eval-restrict"))
+(include-book "parallel/without-waterfall-parallelism" :dir :system)
 
 (local (in-theory (disable gl::generic-geval gl::generic-geval-alt-def
                            gl::geval-for-meta-gtests-nonnil-correct)))
@@ -42,7 +43,6 @@
       (and (consp (car x))
            (atom (caar x))
            (not (gl::g-keyword-symbolp (caar x)))
-           (gl::gobjectp (cdar x))
            (atom-key-gobj-val-alistp (cdr x)))))
 
 (local
@@ -55,42 +55,36 @@
                         (not (g-ite-p x))
                         (not (g-apply-p x))
                         (not (g-var-p x))))
-          :hints(("Goal" :in-theory
-                  (enable* g-concrete-p g-boolean-p g-number-p
-                           g-ite-p g-apply-p g-var-p
-                           g-keyword-symbolp-def)))
+          :hints(("Goal" :in-theory (enable g-keyword-symbolp)))
           :rule-classes ((:rewrite :backchain-limit-lst 0)))
 
-   (defthmd atom-key-gobj-val-alistp-gobjectp
-     (implies (atom-key-gobj-val-alistp x)
-              (gl::gobjectp x))
-     :hints (("goal" :expand ((gl::gobjectp x)
-                              (gl::gobjectp (car x))
-                              (gl::gobjectp (caar x)))
-              :induct (atom-key-gobj-val-alistp x)
-              :in-theory (enable
-                          gl::not-keyword-symbolp-car-impl)))
-     :rule-classes ((:rewrite :backchain-limit-lst 1)
-                    :forward-chaining))
+   ;; (defthmd atom-key-gobj-val-alistp-gobjectp
+   ;;   (implies (atom-key-gobj-val-alistp x)
+   ;;            (gl::gobjectp x))
+   ;;   :hints (("goal" :expand ((gl::gobjectp x)
+   ;;                            (gl::gobjectp (car x))
+   ;;                            (gl::gobjectp (caar x)))
+   ;;            :induct (atom-key-gobj-val-alistp x)
+   ;;            :in-theory (enable
+   ;;                        gl::not-keyword-symbolp-car-impl)))
+   ;;   :rule-classes ((:rewrite :backchain-limit-lst 1)
+   ;;                  :forward-chaining))
 
    (defthm atom-key-gobj-val-alistp-assoc-eval
      (implies (atom-key-gobj-val-alistp x)
               (equal (gl::generic-geval (hons-assoc-equal key x) env)
                      (hons-assoc-equal key (gl::generic-geval x env))))
-     :hints (("goal" :in-theory (e/d (hons-assoc-equal
-                                      atom-key-gobj-val-alistp-gobjectp)
+     :hints (("goal" :in-theory (e/d (hons-assoc-equal)
                                      (suffixp
                                       ;; gl::concrete-key-alistp
                                       member-eq
-                                      gl::gobjectp-cons-case
                                       ;; gl::gobjectp-car-cdr-when-cons
 ; gl::generic-geval-when-g-var-tag
                                       gl::general-number-components-ev))
               :induct (hons-assoc-equal key x))))))
 
 (defun gobj-alist-to-bfr-alist (x hyp)
-  (declare (xargs :guard (and (atom-key-gobj-val-alistp x)
-                              (gl::bfr-p hyp))))
+  (declare (xargs :guard (atom-key-gobj-val-alistp x)))
   (if (atom x)
       (mv nil nil)
     (b* ((test (gl::gtests (cdar x) hyp))
@@ -122,14 +116,14 @@
 (local
  (progn
 
-   (defthm bfr-alistp-gobj-alist-to-bfr-alist
-     (implies (atom-key-gobj-val-alistp x)
-              (gl::bfr-alistp (mv-nth 0 (gobj-alist-to-bfr-alist x hyp)))))
+   ;; (defthm bfr-alistp-gobj-alist-to-bfr-alist
+   ;;   (implies (atom-key-gobj-val-alistp x)
+   ;;            (gl::bfr-alistp (mv-nth 0 (gobj-alist-to-bfr-alist x hyp)))))
    
-   (defthm bfr-p-gobj-alist-to-bfr-alist
-     (implies (and (gl::bfr-p hyp)
-                   (atom-key-gobj-val-alistp x))
-              (gl::bfr-p (mv-nth 1 (gobj-alist-to-bfr-alist x hyp)))))
+   ;; (defthm bfr-p-gobj-alist-to-bfr-alist
+   ;;   (implies (and (gl::bfr-p hyp)
+   ;;                 (atom-key-gobj-val-alistp x))
+   ;;            (gl::bfr-p (mv-nth 1 (gobj-alist-to-bfr-alist x hyp)))))
 
 
    ;; (local (in-theory
@@ -138,19 +132,19 @@
    ;;                  GL-THM::GTESTS-NONNIL-CORRECT-FOR-GLCP-GEVAL
    ;;                  GL-THM::NON-CONSP-EVAL-CORRECT-FOR-GLCP-GEVAL)))
 
-   (defthmd non-keyword-atom-gobjectp
-     (implies (and (not (gl::g-keyword-symbolp x))
-                   (not (consp x)))
-              (gobjectp x))
-     :hints(("Goal" :in-theory (enable gl::gobjectp-def))))
+   ;; (defthmd non-keyword-atom-gobjectp
+   ;;   (implies (and (not (gl::g-keyword-symbolp x))
+   ;;                 (not (consp x)))
+   ;;            (gobjectp x))
+   ;;   :hints(("Goal" :in-theory (enable gl::gobjectp-def))))
 
-   (defthmd gobjectp-cons-case-inverse
-     (implies (and (consp x)
-                   (not (gl::g-keyword-symbolp (car x)))
-                   (not (consp (car x)))
-                   (gobjectp (cdr x)))
-              (gobjectp x))
-     :hints(("Goal" :in-theory (enable gl::gobjectp-def))))
+   ;; (defthmd gobjectp-cons-case-inverse
+   ;;   (implies (and (consp x)
+   ;;                 (not (gl::g-keyword-symbolp (car x)))
+   ;;                 (not (consp (car x)))
+   ;;                 (gobjectp (cdr x)))
+   ;;            (gobjectp x))
+   ;;   :hints(("Goal" :in-theory (enable gl::gobjectp-def))))
 
    (defthm bfr-eval-alist-hons-assoc-equal-iff
      (iff (hons-assoc-equal x (gl::bfr-eval-alist al env))
@@ -165,20 +159,11 @@
    :hints (("goal" :induct (atom-key-gobj-val-alistp x)
             :in-theory (e/d
                (hons-assoc-equal
-                atom-key-gobj-val-alistp-gobjectp
-                gl::not-keyword-symbolp-car-impl
-                non-keyword-atom-gobjectp
-                gobjectp-cons-case-inverse)
+                gl::not-keyword-symbolp-car-impl)
                (member-eq hons-assoc-equal
-                          gl::gobjectp-cons-case
                           gl::general-number-components-ev
                           (:definition atom-key-gobj-val-alistp)
-                          gl::generic-geval-non-gobjectp
-                          gl::general-concrete-obj-correct
-                          ;; gl::general-consp-car-correct
-                          ;;                gl::general-consp-cdr-correct
-; gl::generic-geval-when-g-var-tag
-                          gl::gobj-fix-when-not-gobjectp)))
+                          gl::general-concrete-obj-correct)))
            (and stable-under-simplificationp
                 '(:expand ((:with gl::generic-geval (gl::generic-geval (car x) env))
                            (:with gl::generic-geval (gl::generic-geval x env))
@@ -192,7 +177,6 @@
    (defthm eval-gobj-alist
      (implies (and (atom-key-gobj-val-alistp x)
                    (gl::bfr-eval hyp (car env))
-                   (gl::bfr-p hyp)
                    (not (gl::bfr-eval (mv-nth 1 (gobj-alist-to-bfr-alist x hyp))
                                       (car env))))
               (iff (cdr (hons-assoc-equal
@@ -206,20 +190,11 @@
      :hints (("goal" :in-theory
               (e/d
                (hons-assoc-equal
-                atom-key-gobj-val-alistp-gobjectp
-                gl::not-keyword-symbolp-car-impl
-                non-keyword-atom-gobjectp
-                gobjectp-cons-case-inverse)
+                gl::not-keyword-symbolp-car-impl)
                (member-eq hons-assoc-equal
-                          gl::gobjectp-cons-case
                           gl::general-number-components-ev
                           atom-key-gobj-val-alistp
-                          gl::generic-geval-non-gobjectp
-                          gl::general-concrete-obj-correct
-                          ;; gl::general-consp-car-correct
-                          ;;                gl::general-consp-cdr-correct
-; gl::generic-geval-when-g-var-tag
-                          gl::gobj-fix-when-not-gobjectp))
+                          gl::general-concrete-obj-correct))
               :do-not-induct t
               :expand ((:with gl::generic-geval (gl::generic-geval (car x) env))
                        (:with gl::generic-geval (gl::generic-geval x env))
@@ -242,7 +217,6 @@
    (defthm aig-eval-eval-gobj-alist
      (implies (and (atom-key-gobj-val-alistp x)
                    (gl::bfr-eval hyp (car env))
-                   (gl::bfr-p hyp)
                    (not (gl::bfr-eval 
                          (mv-nth 1 (gobj-alist-to-bfr-alist x hyp))
                          (car env))))
@@ -257,7 +231,6 @@
    (defthm aig-eval-list-eval-gobj-alist
      (implies (and (atom-key-gobj-val-alistp x)
                    (gl::bfr-eval hyp (car env))
-                   (gl::bfr-p hyp)
                    (not (gl::bfr-eval 
                          (mv-nth 1 (gobj-alist-to-bfr-alist x hyp))
                          (car env))))
@@ -268,26 +241,23 @@
                      (aig-eval-list aig (gl::generic-geval x env))))
      :hints (("goal" :induct (aig-eval-list aig (gl::generic-geval x env)))))
 
-   (defthm gobjectp-g-apply
-     (implies (symbolp fn)
-              (equal (gl::gobjectp (gl::g-apply fn args))
-                     (gl::gobjectp args)))
-     :hints (("goal" :in-theory (e/d (gl::gobjectp-def
-                                      gl::g-apply gl::tag
-                                      gl::g-apply-p
-                                      gl::g-apply->fn
-                                      gl::g-apply->args)
-                                     ((force)))
-              :expand ((gl::gobjectp (list* :g-apply fn args))))))
+   ;; (defthm gobjectp-g-apply
+   ;;   (implies (symbolp fn)
+   ;;            (equal (gl::gobjectp (gl::g-apply fn args))
+   ;;                   (gl::gobjectp args)))
+   ;;   :hints (("goal" :in-theory (e/d (gl::gobjectp-def
+   ;;                                    gl::g-apply gl::tag
+   ;;                                    gl::g-apply-p
+   ;;                                    gl::g-apply->fn
+   ;;                                    gl::g-apply->args)
+   ;;                                   ((force)))
+   ;;            :expand ((gl::gobjectp (list* :g-apply fn args))))))
              
 
  
    (local (in-theory (disable atom-key-gobj-val-alistp
                               suffixp ; car-member-when-suffix
-                              faig-bddify-pat
-; gl::g-var-p-implies-gobjectp
-; gl::g-concrete-p-gobjectp1
-                              gobjectp)))))
+                              faig-bddify-pat)))))
 
 
 
@@ -304,15 +274,10 @@
 (local
  (progn
    
-(defthm gobjectp-g-boolean-list
-  (implies (gl::bfr-listp x)
-           (gl::gobjectp (g-boolean-list x)))
-  :hints(("Goal" :in-theory (enable gl::tag))))
 
 (defthm eval-of-g-boolean-list
-  (implies (gl::bfr-listp x)
-           (equal (gl::generic-geval (g-boolean-list x) env)
-                  (gl::bfr-eval-list x (car env))))
+  (equal (gl::generic-geval (g-boolean-list x) env)
+         (gl::bfr-eval-list x (car env)))
   :hints(("Subgoal *1/2"
           :expand ((:free (x) (:with gl::generic-geval
                                      (gl::generic-geval (gl::g-boolean x)
@@ -335,22 +300,22 @@
 
 
 
-(defthmd bfr-alistp-bfr-cases
-  (equal (gl::bfr-alistp x)
-         (gl::bfr-case
-          :bdd (acl2::ubddp-val-alistp x)
-          :aig t))
-  :hints(("Goal" :in-theory (enable bfr-p)
-          :induct (gl::bfr-alistp x))))
+;; (defthmd bfr-alistp-bfr-cases
+;;   (equal (gl::bfr-alistp x)
+;;          (gl::bfr-case
+;;           :bdd (acl2::ubddp-val-alistp x)
+;;           :aig t))
+;;   :hints(("Goal" :in-theory (enable bfr-p)
+;;           :induct (gl::bfr-alistp x))))
 
 
-(defthmd bfr-listp-bfr-cases
-  (equal (gl::bfr-listp x)
-         (gl::bfr-case
-          :bdd (and (acl2::ubdd-listp x) (true-listp x))
-          :aig (true-listp x)))
-  :hints(("Goal" :in-theory (enable gl::bfr-listp gl::bfr-p)
-          :induct (len x))))
+;; (defthmd bfr-listp-bfr-cases
+;;   (equal (gl::bfr-listp x)
+;;          (gl::bfr-case
+;;           :bdd (and (acl2::ubdd-listp x) (true-listp x))
+;;           :aig (true-listp x)))
+;;   :hints(("Goal" :in-theory (enable gl::bfr-listp gl::bfr-p)
+;;           :induct (len x))))
 
 (defun aig-bfr-compose (x al)
   (gl::bfr-case
@@ -387,45 +352,36 @@
          (aig-eval-list aigs (bdd-eval-alst bdd-al env)))
   :hints(("Goal" :in-theory (enable aig-q-compose-correct))))
 
+(defcong bdd-equiv-list equal (eval-bdd-list x env) 1)
+
 (defthm aig-bfrify-list-ok
-  (implies
-   (gl::bfr-alistp bfr-al)
-   (mv-let (res exact)
-     (aig-bfrify-list tries aigs bfr-al maybe-wash-args)
-     (implies exact
-              (equal (bfr-eval-list res env)
-                     (aig-eval-list aigs
-                                    (gl::bfr-eval-alist bfr-al env))))))
+  (mv-let (res exact)
+    (aig-bfrify-list tries aigs bfr-al maybe-wash-args)
+    (implies exact
+             (equal (bfr-eval-list res env)
+                    (aig-eval-list aigs
+                                   (gl::bfr-eval-alist bfr-al env)))))
   :hints(("Goal" :in-theory (enable bfr-eval-list-bfr-cases
                                     gl::bfr-eval-alist-bfr-cases
-                                    bfr-alistp-bfr-cases
-                                    aig-q-compose-correct)
+                                    aig-q-compose-correct
+                                    aig-bddify-list-ok)
           :induct (len aigs))))
 
+ 
 
 
-
-(defthm aig-bfrify-list-bfr-listp
-  (implies (gl::bfr-alistp bfr-al)
-           (gl::bfr-listp (mv-nth 0 (aig-bfrify-list tries aigs bfr-al maybe-wash-args))))
-  :hints (("goal" :in-theory (enable bfr-listp-bfr-cases bfr-alistp-bfr-cases))))
+;; (defthm aig-bfrify-list-bfr-listp
+;;   (implies (gl::bfr-alistp bfr-al)
+;;            (gl::bfr-listp (mv-nth 0 (aig-bfrify-list tries aigs bfr-al maybe-wash-args))))
+;;   :hints (("goal" :in-theory (enable bfr-listp-bfr-cases bfr-alistp-bfr-cases))))
 
 (in-theory (disable aig-bfrify-list))
 
 (defun aig-eval-list-symbolic
   (x al tries maybe-wash-args hyp clk)
-  (declare (xargs :guard (and (gl::gobjectp x)
-                              (gl::gobjectp al)
-                              (gl::gobjectp tries)
-                              (gl::bfr-p hyp))
-                  :guard-hints (("goal" :in-theory
-                                 (e/d (gl::tag) ((force)
-                                                 aig-bddify-list)))))
+  (declare (xargs :guard t)
            (ignore clk))
-  (let ((x (mbe :logic (gl::gobj-fix x) :exec x))
-        (al (mbe :logic (gl::gobj-fix al) :exec al))
-        (hyp (mbe :logic (gl::bfr-fix hyp) :exec hyp))
-        (tries (if (gl::general-concretep tries)
+  (let ((tries (if (gl::general-concretep tries)
                    (gl::general-concrete-obj tries)
                  (er hard? 'aig-eval-list-symbolic "Expected tries to be concrete~%"))))
     (if (and (atom-key-gobj-val-alistp al)
@@ -469,47 +425,44 @@
 
 
 
-(local
- (defthm gobjectp-aig-eval-list-symbolic
-   (gl::gobjectp (aig-eval-list-symbolic x al tries maybe-wash-args
-                                         hyp clk))
-     :hints(("Goal" :in-theory (disable member-eq
-                                        atom-key-gobj-val-alistp
-                                        member-equal-of-cons
-                                        gl::gobjectp-cons-case
-                                        aig-bddify-list)))))
-
+;; (local
+;;  (defthm gobjectp-aig-eval-list-symbolic
+;;    (gl::gobjectp (aig-eval-list-symbolic x al tries maybe-wash-args
+;;                                          hyp clk))
+;;      :hints(("Goal" :in-theory (disable member-eq
+;;                                         atom-key-gobj-val-alistp
+;;                                         member-of-cons
+;;                                         gl::gobjectp-cons-case
+;;                                         aig-bddify-list)))))
 
 (make-event
  `(defun ,(gl-fnsym 'aig-eval-list-with-bddify)
     (x al tries maybe-wash-args hyp clk)
-    (declare (xargs :guard (and (gl::gobjectp x)
-                                (gl::gobjectp al)
-                                (gl::gobjectp tries)
-                                (gl::bfr-p hyp))))
+    (declare (xargs :guard t))
     (aig-eval-list-symbolic x al tries maybe-wash-args hyp clk)))
 
 
-(defthm gobjectp-g-aig-eval-list
-  (gl::gobjectp (glr aig-eval-list-with-bddify
-                     x al tries maybe-wash-args hyp clk))
-  :hints(("Goal" :in-theory (disable aig-eval-list-symbolic))))
+;; (defthm gobjectp-g-aig-eval-list
+;;   (gl::gobjectp (glr aig-eval-list-with-bddify
+;;                      x al tries maybe-wash-args hyp clk))
+;;   :hints(("Goal" :in-theory (disable aig-eval-list-symbolic))))
+
+;; (add-to-ruleset! gl::g-gobjectp-lemmas '(gobjectp-g-aig-eval-list))
 
-(add-to-ruleset! gl::g-gobjectp-lemmas '(gobjectp-g-aig-eval-list))
 
 
 (encapsulate nil
   (make-event 
-   `(gl::defapply aig-eval-ap
-                  ,(list* 'aig-eval-list
-                          (cdar (table-alist 'gl::g-apply-table (w state)))))))
-
-(encapsulate nil
-  (gl::def-eval-g aig-eval-ev aig-eval-ap))
+   `(acl2::without-waterfall-parallelism
+     (gl::def-eval-g aig-eval-ev
+                     ,(list* 'aig-eval-list
+                             (cdar (table-alist 'gl::g-apply-table (w state))))))))
 
 (local
  (progn
-   (gl::correctness-lemmas aig-eval-ev)
+
+   (acl2::without-waterfall-parallelism 
+    (gl::correctness-lemmas aig-eval-ev))
 
    (gl::eval-g-prove-f-i aig-eval-ev-f-i-generic-geval aig-eval-ev
                          gl::generic-geval)
@@ -532,21 +485,21 @@
 
 (in-theory (disable aig-bddify-list aig-eval-list))
 
-(defun def-g-correctness-thm-fn (thmname fn eval hints world)
-  (declare (xargs :mode :program))
-  (b* ((formals (fgetprop fn 'formals t world)))
-    `(progn (defthm ,thmname
-              (implies ,'(bfr-eval gl::hyp (car gl::env))
-                       (equal (,eval (glr ,fn ,@formals gl::hyp gl::clk)
-                                     gl::env)
-                              (,fn . ,(gl::eval-list-env eval formals))))
-              :hints ,hints)
-            (table gl::gl-function-info ',fn `(,(gl-fnsym ',fn)
-                                               (,',thmname . ,',eval))))))
+;; (defun def-g-correctness-thm-fn (thmname fn eval hints world)
+;;   (declare (xargs :mode :program))
+;;   (b* ((formals (fgetprop fn 'formals t world)))
+;;     `(progn (defthm ,thmname
+;;               (implies ,'(bfr-eval gl::hyp (car gl::env))
+;;                        (equal (,eval (glr ,fn ,@formals gl::hyp gl::clk)
+;;                                      gl::env)
+;;                               (,fn . ,(gl::eval-list-env eval formals))))
+;;               :hints ,hints)
+;;             (table gl::gl-function-info ',fn `(,(gl-fnsym ',fn)
+;;                                                (,',thmname . ,',eval))))))
 
-(defmacro def-g-correctness-thm (thmname fn eval &key hints)
-  `(make-event (def-g-correctness-thm-fn
-                 ',thmname ',fn ',eval ',hints (w state))))
+;; (defmacro def-g-correctness-thm (thmname fn eval &key hints)
+;;   `(make-event (def-g-correctness-thm-fn
+;;                  ',thmname ',fn ',eval ',hints (w state))))
 
 ;; (local
 ;;  (defthm eval-bdd-list-aig-q-compose-list
@@ -564,22 +517,17 @@
                                 env)
                    (aig-eval-list (aig-eval-ev x env)
                                   (aig-eval-ev al env))))
-   :hints (("goal" :in-theory (e/d (eval-bdd-list-aig-q-compose-list
-                                    gl::bfr-eval-of-bfr-fix gl::gobj-fix)
+   :hints (("goal" :in-theory (e/d (eval-bdd-list-aig-q-compose-list)
                                    (member-eq atom-key-gobj-val-alistp
                                               member-equal eval-bdd-list
-                                              gl::gobj-fix-when-gobjectp
-                                              gl::gobj-fix-when-not-gobjectp
                                               gl-thm::generic-geval-g-boolean-for-aig-eval-ev
-                                              aig-bddify-list
-                                              gl::gobj-fix))
+                                              aig-bddify-list))
             :do-not-induct t))))
 
-
-(def-g-correctness-thm g-aig-eval-list-with-bddify-correct
+(gl::def-g-correct-thm ;;  g-aig-eval-list-with-bddify-correct
   aig-eval-list-with-bddify aig-eval-ev
-  :hints (("goal" :in-theory (e/d (aig-eval-list-with-bddify)
-                                  (aig-eval-list-symbolic)))))
+  :hints `(("goal" :in-theory (e/d (aig-eval-list-with-bddify)
+                                   (aig-eval-list-symbolic)))))
 
 ;; The theorems that we'll set as the preferred defs for the following
 ;; functions express each of these functions in terms of aig-eval-list.
