@@ -19,12 +19,12 @@
 ; Original author: Jared Davis <jared@centtech.com>
 
 (in-package "ACL2")
-(include-book "cutil/defsection" :dir :system)
+(include-book "xdoc/top" :dir :system)
 (include-book "tools/bstar" :dir :system)
 (include-book "finite-set-theory/osets/sets" :dir :system)
 (local (include-book "ihs-extensions"))
-(local (include-book "unicode/rev" :dir :system))
-(local (include-book "unicode/append" :dir :system))
+(local (include-book "std/lists/rev" :dir :system))
+(local (include-book "std/lists/append" :dir :system))
 ; (local (include-book "arithmetic-3/extra/top-ext" :dir :system))
 
 (local (in-theory (enable* arith-equiv-forwarding)))
