@@ -24,9 +24,9 @@
 (local (include-book "equal-by-logbitp"))
 (local (include-book "ihs-extensions"))
 (local (include-book "../misc/equal-sets"))
-(local (include-book "unicode/revappend" :dir :system))
-(local (include-book "unicode/rev" :dir :system))
-(local (include-book "unicode/append" :dir :system))
+(local (include-book "std/lists/revappend" :dir :system))
+(local (include-book "std/lists/rev" :dir :system))
+(local (include-book "std/lists/append" :dir :system))
 (local (include-book "arithmetic-3/bind-free/top" :dir :system))
 (local (include-book "ihs/quotient-remainder-lemmas" :dir :system))
 (local (in-theory (disable cancel-mod-+)))
@@ -206,7 +206,7 @@ optimized version of @(see sbitset-pair-members).</p>"
                            expt-is-increasing-for-base->-1
                            expt-2-monotonic)))
 
-(local (in-theory (disable assoc-append)))
+(local (in-theory (disable associativity-of-append)))
 
 (local (defthm integer-listp-of-append
          (implies (and (integer-listp x)
