@@ -22,8 +22,8 @@
 (include-book "misc/definline" :dir :system)
 (include-book "str/cat" :dir :system)
 (include-book "str/natstr" :dir :system)
-(include-book "unicode/two-nats-measure" :dir :system)
-(local (include-book "unicode/explode-atom" :dir :system))
+(include-book "std/misc/two-nats-measure" :dir :system)
+(local (include-book "std/misc/explode-atom" :dir :system))
 (local (include-book "centaur/bitops/ihsext-basics" :dir :system))
 (local (include-book "misc/assert" :dir :system))
 
@@ -97,7 +97,7 @@ identical JSON encodings:</p>
 <li>Why use \"NIL\" and \"T\" instead of, e.g., JSON's null (or false) and
 true?  The main reason is that null/true can't be used as keys in JSON objects,
 so this would lead to either special handling of alists with nil/t keys, or
-alists that have different encoding depending on theri keys, and it just seems
+alists that have different encoding depending on their keys, and it just seems
 simpler to use strings for all atoms.</li>
 
 <li>Why use \"123\" when JSON has integers?  The same reason as for NIL and T
@@ -299,7 +299,7 @@ character list."
                              (+ 1 (mbe :logic (nfix n) :exec n))
                              xl acc))))
 
-  (local (include-book "unicode/nthcdr" :dir :system))
+  (local (include-book "std/lists/nthcdr" :dir :system))
   (local (include-book "arithmetic/top" :dir :system))
 
   (local (defthm nthcdr-of-increment
@@ -359,8 +359,7 @@ character list."
   (local (in-theory (enable json-encode-atom)))
 
   (defthm character-listp-of-json-encode-atom
-    (implies (and (atom x)
-                  (character-listp acc))
+    (implies (character-listp acc)
              (character-listp (json-encode-atom x acc))))
 
   (local (defun test (x)
@@ -544,7 +543,8 @@ to fix up atoms.</p>"
        :flag json-encode-main)
      (defthm l1
        (implies (and (character-listp acc)
-                     (json-simple-alist-p x))
+                     ;(json-simple-alist-p x)
+                     )
                 (character-listp (json-encode-simple-alist x acc)))
        :flag json-encode-simple-alist)
      (defthm l2
@@ -564,7 +564,19 @@ to fix up atoms.</p>"
 
   (defthm character-listp-of-json-encode-main
     (implies (character-listp acc)
-             (character-listp (json-encode-main x acc)))))
+             (character-listp (json-encode-main x acc))))
+
+  (defthm character-listp-of-json-encode-simple-alist
+    (implies (character-listp acc)
+             (character-listp (json-encode-simple-alist x acc))))
+
+  (defthm character-listp-of-json-encode-true-list
+    (implies (character-listp acc)
+             (character-listp (json-encode-true-list x acc))))
+
+  (defthm character-listp-of-json-encode-improper-cons-list
+    (implies (character-listp acc)
+             (character-listp (json-encode-improper-cons-list x acc)))))
 
 
 (defsection json-encode
