@@ -24,11 +24,23 @@
 (include-book "centaur/esim/plist" :dir :system)
 (include-book "centaur/misc/patterns" :dir :system)
 (include-book "centaur/4v-sexpr/sexpr-vars" :dir :system)
-(include-book "unicode/explode-nonnegative-integer" :dir :system)
-(include-book "unicode/two-nats-measure" :dir :system)
+(include-book "std/misc/explode-nonnegative-integer" :dir :system)
+(include-book "std/misc/two-nats-measure" :dir :system)
 (include-book "arithmetic/nat-listp" :dir :system)
 (set-well-founded-relation nat-list-<)
 
+(make-event
+
+; Disabling waterfall parallelism because this book allegedly uses memoization
+; while performing its proofs.
+
+ (if (and (hons-enabledp state) 
+          (f-get-global 'parallel-execution-enabled state)) 
+     (er-progn (set-waterfall-parallelism nil)
+               (value '(value-triple nil)))
+   (value '(value-triple nil))))
+
+
 (defund stringify-atom (x)
   (declare (xargs :guard (atom x)))
   (cond ((symbolp x) (symbol-name x))
@@ -121,7 +133,7 @@
     (pat-flatten (gpl key (car occs))
                  (collect-signal-list key (cdr occs)))))
 
-(memoize 'collect-signal-list :inline nil)
+(memoize 'collect-signal-list :recursive nil)
 
 (defabbrev driven-signals (mod)
   (collect-signal-list :o (gpl :occs mod)))
