@@ -31,11 +31,21 @@
 (include-book "cutil/defmvtypes" :dir :system)
 (include-book "cutil/defprojection" :dir :system)
 (include-book "str/natstr" :dir :system)
-(include-book "unicode/final-cdr" :dir :system)
+(include-book "std/lists/final-cdr" :dir :system)
 (include-book "centaur/vl/util/defs" :dir :system)
 (local (include-book "../esim-sexpr-support-thms"))
 (local (include-book "centaur/vl/util/arithmetic" :dir :system))
 
+(make-event
+
+; Disabling waterfall parallelism because this book allegedly uses memoization
+; while performing its proofs.  
+
+ (if (and (hons-enabledp state) 
+          (f-get-global 'parallel-execution-enabled state)) 
+     (er-progn (set-waterfall-parallelism nil)
+               (value '(value-triple nil)))
+   (value '(value-triple nil))))
 
 (local (defthm atom-listp-of-append
          (implies (and (atom-listp x)
@@ -48,10 +58,10 @@
          :hints(("Goal" :in-theory (e/d (pat-flatten1)
                                         ((force)))))))
 
-(local (defthm alist-keys-of-pairlis$
-         (equal (alist-keys (pairlis$ x y))
-                (list-fix x))
-         :hints(("Goal" :in-theory (enable pairlis$)))))
+;; (local (defthm alist-keys-of-pairlis$
+;;          (equal (alist-keys (pairlis$ x y))
+;;                 (list-fix x))
+;;          :hints(("Goal" :in-theory (enable pairlis$)))))
 
 (local (defthm consp-when-member-of-atom-listp
          (implies (and (atom-listp x)
