@@ -4,7 +4,7 @@
 
 
 (include-book "parallel/without-waterfall-parallelism" :dir :system)
-(include-book "centaur/misc/defapply" :dir :system)
+;; (include-book "centaur/misc/defapply" :dir :system)
 (include-book "gify")
 (local (include-book "gify-thms"))
 (include-book "run-gified-cp")
@@ -17,10 +17,10 @@
 ;; pre-Gified functions.
 
 
-(defthm gobjectp-nth-gobject-listp
-  (implies (gobject-listp lst)
-           (gobjectp (nth n lst)))
-  :hints(("Goal" :in-theory (enable gobject-listp))))
+;; (defthm gobjectp-nth-gobject-listp
+;;   (implies (gobject-listp lst)
+;;            (gobjectp (nth n lst)))
+;;   :hints(("Goal" :in-theory (enable gobject-listp))))
 
 (defun glcp-predef-cases-fn (names world)
   (declare (Xargs :mode :program))
@@ -55,43 +55,37 @@
 
 
 
-(defthmd gl-eval-car-cdr-of-gobject-listp
-  (implies (gobject-listp x)
+(defthmd gl-eval-car-cdr-of-gobj-listp
+  (implies (gobj-listp x)
            (and (equal (car (generic-geval x env))
                        (generic-geval (car x) env))
                 (equal (cdr (generic-geval x env))
                        (generic-geval (cdr x) env))))
   :hints (("goal" :expand ((:with generic-geval (generic-geval x env)))
-           :in-theory (e/d* (gobjectp-car-impl-not-g-types
-                             gobject-listp-impl-gobjectp
-                             gobject-listp)
+           :in-theory (e/d* (gobj-listp)
                             (general-consp-car-correct
                              general-consp-cdr-correct)))))
 
-(defthmd gl-eval-car-gobjectp
-  (implies (and (gobjectp x)
-                (consp x)
-                (gobjectp (car x)))
-           (equal (generic-geval x env)
-                  (cons (generic-geval (car x) env)
-                        (generic-geval (cdr x) env))))
-  :hints (("goal" :expand ((generic-geval x env))
-           :in-theory (enable gobjectp-car-impl-not-g-types))))
+;; (defthmd gl-eval-car-gobjectp
+;;   (implies (consp x)
+;;            (equal (generic-geval x env)
+;;                   (cons (generic-geval (car x) env)
+;;                         (generic-geval (cdr x) env))))
+;;   :hints (("goal" :expand ((generic-geval x env))
+;;            :in-theory (enable gobjectp-car-impl-not-g-types))))
 
 (defthmd gl-eval-of-nil
   (equal (generic-geval nil env) nil))
 
 
 
-(defthm gl-eval-consp-when-gobject-listp
-  (implies (gobject-listp x)
+(defthm gl-eval-consp-when-gobj-listp
+  (implies (gobj-listp x)
            (equal (consp (generic-geval x env))
                   (consp x)))
   :hints (("goal" :expand ((:with generic-geval (generic-geval x env))
-                           (gobject-listp x))
-           :in-theory (e/d (gobjectp-car-impl-not-g-types
-                            gobject-listp-impl-gobjectp
-                            gobject-listp)
+                           (gobj-listp x))
+           :in-theory (e/d (gobj-listp)
                            (general-consp-car-correct
                             general-consp-cdr-correct)))))
 
@@ -112,10 +106,10 @@
 ;;   (equal (cdr (acl2::kwote-lst lst))
 ;;          (acl2::kwote-lst (cdr lst))))
  
-;; (defthm gobject-listp-cdr
-;;   (implies (gobject-listp x)
-;;            (gobject-listp (cdr x)))
-;;   :hints(("Goal" :in-theory (enable gobject-listp))))
+;; (defthm gobj-listp-cdr
+;;   (implies (gobj-listp x)
+;;            (gobj-listp (cdr x)))
+;;   :hints(("Goal" :in-theory (enable gobj-listp))))
 
 
 
@@ -239,68 +233,36 @@
         (cons (car fns)
               (collect-non-fns (cdr fns) world))
       (collect-non-fns (cdr fns) world))))
+
+(defthm gobj-listp-true-listp
+  (implies (gobj-listp x)
+           (true-listp x))
+  :hints(("Goal" :in-theory (enable gobj-listp)))
+  :rule-classes :forward-chaining)
       
 (defun def-gl-clause-processor-fn
-  (clause-proc apply-fns include-nonrec top-apply-fns output state)
+  (clause-proc output state)
   (declare (xargs :mode :program :stobjs state))
   (b* ((world (w state))
-       (non-fns (append (collect-non-fns apply-fns world)
-                        (collect-non-fns top-apply-fns world)))
-       ((when non-fns)
-        (er hard? 'def-gl-clause-processor
-            "The following symbols are not functions: ~x0~%" non-fns))
        (current-geval (find-current-geval world))
        (geval (or current-geval
                   (incat clause-proc (symbol-name clause-proc) "-GEVAL")))
        (run-gified (incat clause-proc (symbol-name clause-proc) "-RUN-GIFIED"))
-       (run-gified-inner (incat clause-proc (symbol-name clause-proc) "-RUN-GIFIED-INNER"))
-       (apply-concrete-inner (incat clause-proc (symbol-name clause-proc)
-                                    "-APPLY-INNER"))
-       (apply-concrete (incat clause-proc (symbol-name clause-proc)
-                              "-APPLY"))
-       (apply-concrete-inner-lemma
-        (incat clause-proc (symbol-name clause-proc)
-               "-APPLY-INNER-CORRECT"))
-       (apply-concrete-lemma
-        (incat clause-proc (symbol-name clause-proc) "-APPLY-CORRECT"))
-       (apply-concrete-state-p1
-        (incat clause-proc (symbol-name clause-proc) "-STATE-P1"))
        (g-fns (strip-cars (table-alist 'gl-function-info world)))
-       ((mv done collected-apply-fns)
-        (collect-fns-list apply-fns nil nil world))
-       (- (flush-hons-get-hash-table-link done))
-       (recursive-apply-fns
-        (if include-nonrec
-            collected-apply-fns
-          (filter-recursive-fns collected-apply-fns world)))
-       (auto-apply-fns (gl-clause-proc-auto-exec-fns))
-       (forbid-apply-fns (gl-clause-proc-forbidden-exec-fns))
-       (final-apply-fns
-        (acl2::hons-remove-duplicates
-         (acl2::hons-set-diff
-          (append top-apply-fns apply-fns recursive-apply-fns auto-apply-fns)
-          ;; no need to have g-fns in the apply
-          (append g-fns forbid-apply-fns))))
-       (- (cw "number of applyable functions: ~x0~%"
-              (len final-apply-fns)))
        (ev (incat clause-proc (symbol-name clause-proc) "-EV"))
        (ev-lst (incat clause-proc (symbol-name clause-proc) "-EV-LST"))
        (falsify (incat clause-proc (symbol-name clause-proc) "-EV-FALSIFY"))
+       (badguy (incat clause-proc (symbol-name clause-proc)
+                      "-EV-META-EXTRACT-GLOBAL-BADGUY"))
+       (meta-facts
+        (incat clause-proc (symbol-name clause-proc) "-EV-META-EXTRACT-GLOBAL-FACTS"))
        (ctrex-thm (incat clause-proc (symbol-name clause-proc)
                          "-EV-FALSIFY-COUNTEREXAMPLE"))
        (constraints (incat ev (symbol-name ev) "-CONSTRAINTS"))
        (f-i-thm (incat geval (symbol-name geval)
                        "-IS-FUNCTIONAL-INST-OF-GENERIC-GEVAL-FOR-GL-CLAUSE-PROC"))
-       (run-gified-inner-correct
-        (incat run-gified (symbol-name run-gified) "-INNER-CORRECT"))
        (run-gified-correct
         (incat run-gified (symbol-name run-gified) "-CORRECT"))
-       (run-gified-inner-gobjectp
-        (incat run-gified (symbol-name run-gified) "-INNER-GOBJECTP"))
-       (run-gified-gobjectp
-        (incat run-gified (symbol-name run-gified) "-GOBJECTP"))
-       (run-gified-state-p1
-        (incat run-gified (symbol-name run-gified) "-STATE-P1"))
        (interp-term (interp-term-fnname clause-proc))
        (interp-list (incat clause-proc (symbol-name clause-proc)
                            "-INTERP-LIST"))
@@ -314,9 +276,9 @@
                 (run-cases . ,run-cases)
                 (clause-proc-name . ',clause-proc)
                 (clause-proc . ,clause-proc)
-                (geval-name . ',geval)
                 (run-gified . ,run-gified)
-                (apply-concrete . ,apply-concrete)))
+                ;; (apply-concrete . ,apply-concrete)
+                ))
        (f-i-lemmas (incat clause-proc (symbol-name clause-proc)
                           "-FUNCTIONAL-INSTANCE-LEMMAS"))
        (correct-thm (incat clause-proc (symbol-name clause-proc) "-CORRECT")))
@@ -333,35 +295,20 @@
                           :output nil)))
         (encapsulate nil
           (set-case-split-limitations '(1 1))
-          (defun ,run-gified-inner
-            (fn actuals hyp clk)
-            (declare (xargs :guard (and (symbolp fn)
-                                        (gobject-listp actuals)
-                                        (bfr-p hyp)
-                                        (natp clk))
-                            :guard-hints
-                            (("goal" :in-theory
-                              (e/d** (gobjectp-nth-gobject-listp
-                                      (:forward-chaining gobject-listp-true-listp)))
-                              :do-not '(preprocess)))))
-            (case fn
-              . ,(glcp-predef-cases-fn
-                  (remove 'if g-fns) world)))
           (defun ,run-gified
             (fn actuals hyp clk state)
             (declare (xargs :guard (and (symbolp fn)
-                                        (gobject-listp actuals)
-                                        (bfr-p hyp)
+                                        (gobj-listp actuals)
                                         (natp clk))
                             :guard-hints
                             (("goal" :in-theory
-                              (e/d** (gobjectp-nth-gobject-listp
-                                      (:forward-chaining gobject-listp-true-listp)))
+                              (e/d** ((:forward-chaining gobj-listp-true-listp)))
                               :do-not '(preprocess)))
-                            :stobjs state))
-            (mv-let (ok ans)
-              (,run-gified-inner fn actuals hyp clk)
-              (mv ok ans state))))
+                            :stobjs state)
+                     (ignorable state))
+            (case fn
+              . ,(glcp-predef-cases-fn
+                  (remove 'if g-fns) world))))
 
        
 
@@ -371,69 +318,27 @@
          ,ev ,ev-lst
          ,(fns-to-calls
            (append `(if gl-cp-hint shape-spec-obj-in-range
-                      return-last use-by-hint equal not cons gl-aside
-                      gl-ignore
-                      gl-error)
+                      return-last use-by-hint equal acl2::typespec-check implies iff
+                      not cons gl-aside gl-ignore gl-error)
                    (set-difference-eq
-                    (union-eq g-fns final-apply-fns)
+                    g-fns
                     `(if gl-cp-hint shape-spec-obj-in-range
                        return-last use-by-hint equal not cons
-                       ,@*forbidden-apply-functions*
                        ,geval gl-aside gl-ignore gl-error)))
-           world))
+           world)
+         :namedp t)
         (local (def-ruleset! ,constraints
                  (set-difference-theories
                   (current-theory :here)
                   (current-theory 'dummy-label-for-make-evaluator-fn))))
-        (defchoose ,falsify (a) (x)
-          (not (,ev x a)))
+        (acl2::def-meta-extract ,ev ,ev-lst)
+        ;; (defchoose ,falsify (a) (x)
+        ;;   (not (,ev x a)))
         (local (defthm ,ctrex-thm
                  (implies (not (,ev x a))
                           (not (,ev x (,falsify x))))
                  :hints (("goal" :use ,falsify))))
 
-        (encapsulate
-          nil
-          (local (in-theory nil))
-          ;; Define the apply-concrete function
-          (acl2::defapply ,apply-concrete-inner ,final-apply-fns
-                          :theoremsp nil)
-          (defun ,apply-concrete (acl2::fn args state)
-            (declare (xargs :guard (true-listp args)
-                            :stobjs state))
-            (mv-let (ok ans)
-              (,apply-concrete-inner acl2::fn args)
-              (mv ok ans state)))
-                            
-
-          (local (defthm ,apply-concrete-inner-lemma
-                   (implies (mv-nth 0 (,apply-concrete-inner acl2::fn args))
-                            (equal (mv-nth 1 (,apply-concrete-inner acl2::fn args))
-                                   (,ev (cons acl2::fn (acl2::kwote-lst args))
-                                        nil)))
-                   :hints (("goal" :clause-processor
-                            (acl2::apply-for-ev-cp clause nil state))
-                           (acl2::use-by-computed-hint clause)
-                           (acl2::use-these-hints-hint clause))))
-          (defthm ,apply-concrete-lemma
-            (implies (mv-nth 0 (,apply-concrete acl2::fn args state))
-                     (equal (mv-nth 1 (,apply-concrete acl2::fn args
-                                                       state))
-                            (,ev (cons acl2::fn (acl2::kwote-lst
-                                                 args))
-                                 nil)))
-            :hints(("Goal" :in-theory (e/d** (,apply-concrete
-                                              ,apply-concrete-inner-lemma
-                                              car-cons cdr-cons)))))
-
-          (defthm ,apply-concrete-state-p1
-            (implies (state-p1 state)
-                     (state-p1 (mv-nth 2 (,apply-concrete acl2::fn args
-                                                       state))))
-            :hints(("Goal" :in-theory (e/d** (,apply-concrete)
-                                             (state-p1 ,apply-concrete-inner))))))
-
-        (in-theory (disable ,apply-concrete))
 
         ;; Define the interpreter mutual-recursion, the
         ;; run-parametrized and run-cases functions, and the clause proc.
@@ -447,39 +352,18 @@
          (progn
            (eval-g-prove-f-i ,f-i-thm ,geval generic-geval)
            (eval-g-functional-instance
-            gl-eval-car-cdr-of-gobject-listp ,geval generic-geval)
+            gl-eval-car-cdr-of-gobj-listp ,geval generic-geval)
            (eval-g-functional-instance
-            gl-eval-consp-when-gobject-listp ,geval generic-geval)
+            gl-eval-consp-when-gobj-listp ,geval generic-geval)
            (eval-g-functional-instance
             gl-eval-of-nil ,geval generic-geval)
            (eval-g-functional-instance
             general-concrete-obj-correct ,geval generic-geval)
            
            ;; Prove correctness of run-gified
-           (defthm ,run-gified-inner-correct
-             (implies (and (bfr-eval hyp (car env))
-                           (gobject-listp actuals)
-                           (mv-nth 0 (,run-gified-inner
-                                      fn actuals hyp clk)))
-                      (equal (,geval (mv-nth 1 (,run-gified-inner
-                                                fn actuals hyp clk))
-                                     env)
-                             (,ev (cons fn (acl2::kwote-lst
-                                            (,geval actuals env))) nil)))
-             :hints (("goal" :clause-processor
-                      (run-gified-clause-proc
-                       clause
-                       '(,(f-i-thmname
-                           'gl-eval-consp-when-gobject-listp geval)
-                         ,(f-i-thmname
-                           'gl-eval-car-cdr-of-gobject-listp geval)
-                         ,(f-i-thmname 'gl-eval-of-nil geval))
-                       state))
-                     (use-by-computed-hint clause)))
-
            (defthm ,run-gified-correct
              (implies (and (bfr-eval hyp (car env))
-                           (gobject-listp actuals)
+                           (gobj-listp actuals)
                            (mv-nth 0 (,run-gified
                                       fn actuals hyp clk state)))
                       (equal (,geval (mv-nth 1 (,run-gified
@@ -487,36 +371,16 @@
                                      env)
                              (,ev (cons fn (acl2::kwote-lst
                                             (,geval actuals env))) nil)))
-             :hints (("goal" :in-theory (e/d** (,run-gified-inner-correct
-                                                ,run-gified
-                                                mv-nth-cons-meta)
-                                               (,run-gified-inner)))))
-
-           ;; Prove GOBJECTP of run-gified
-           (defthm ,run-gified-inner-gobjectp
-             (gobjectp (mv-nth 1 (,run-gified-inner
-                                  fn actuals hyp clk)))
-             :hints (("goal" :in-theory
-                      (e/d** ((:ruleset g-gobjectp-lemmas)
-                              ,run-gified-inner
-                              (:compound-recognizer
-                               booleanp-compound-recognizer))))))
-
-           (defthm ,run-gified-gobjectp
-             (gobjectp (mv-nth 1 (,run-gified
-                                  fn actuals hyp clk state)))
-             :hints (("goal" :in-theory
-                      (e/d** (,run-gified
-                              ,run-gified-inner-gobjectp)
-                             (,run-gified-inner)))))
-
-           (defthm ,run-gified-state-p1
-             (implies (state-p1 state)
-                      (state-p1 (mv-nth 2 (,run-gified
-                                           fn actuals hyp clk state))))
-             :hints (("goal" :in-theory (e/d** (,run-gified)
-                                               (,run-gified-inner)))))
-             
+             :hints (("goal" :clause-processor
+                      (run-gified-clause-proc
+                       clause
+                       '(,(f-i-thmname
+                           'gl-eval-consp-when-gobj-listp geval)
+                         ,(f-i-thmname
+                           'gl-eval-car-cdr-of-gobj-listp geval)
+                         ,(f-i-thmname 'gl-eval-of-nil geval))
+                       state))
+                     (use-by-computed-hint clause)))
 
            (in-theory (disable ,run-gified))
 
@@ -527,7 +391,7 @@
            (eval-g-functional-instance mk-g-boolean-correct
                                        ,geval generic-geval)
            (eval-g-functional-instance
-            gl-eval-car-gobjectp ,geval generic-geval)
+            generic-geval-gl-cons ,geval generic-geval)
 
            (eval-g-functional-instance
             gobj-to-param-space-correct ,geval generic-geval)
@@ -536,21 +400,16 @@
             generic-geval-non-cons ,geval generic-geval)
 
            (def-ruleset! ,f-i-lemmas
-             (append '(car-cons cdr-cons
-; [Removed by Matt K. to handle changes to member, assoc, etc. after ACL2 4.2.]
-;                                 acl2::no-duplicatesp-is-no-duplicatesp-equal
-                                )
-                     (let ((constr (acl2::ruleset ',constraints)))
-                       (nthcdr (- (len constr) 18) constr))
+             (append '(car-cons cdr-cons)
+                     ;; (let ((constr (acl2::ruleset ',constraints)))
+                     ;;   (nthcdr (- (len constr) 18) constr))
                      '(,ctrex-thm
                        ,run-gified-correct
-                       ,run-gified-state-p1
-                       ,run-gified-gobjectp
-                       ,apply-concrete-lemma
-                       ,apply-concrete-state-p1
-                       ,(f-i-thmname 'gl-eval-car-gobjectp geval)
+                       ;; ,apply-concrete-lemma
+                       ;; ,apply-concrete-state
+                       ,(f-i-thmname 'generic-geval-gl-cons geval)
                        (:type-prescription ,run-gified)
-                       (:type-prescription ,apply-concrete)
+                       ;; (:type-prescription ,apply-concrete)
                        ,(f-i-thmname 'gobj-ite-merge-correct geval)
                        ,(f-i-thmname 'gtests-nonnil-correct geval)
                        ,(f-i-thmname 'gtests-obj-correct geval)
@@ -560,7 +419,6 @@
                        ,(f-i-thmname 'g-concrete-quote-correct geval)
                        ,(f-i-thmname 'mk-g-ite-correct geval)
                        ,(f-i-thmname 'generic-geval-non-cons geval)
-                       ,(f-i-thmname 'generic-geval-gobj-fix geval)
                        ,(f-i-thmname 'gobj-to-param-space-correct geval)
                        ,(f-i-thmname 'general-concrete-obj-correct geval))))))
 
@@ -577,26 +435,29 @@
                    (glcp-generic-ev-lst ,ev-lst)
                    (glcp-generic-geval ,geval)
                    (glcp-generic-run-gified ,run-gified)
-                   (glcp-generic-run-gified-guard-wrapper ,run-gified)
                    (glcp-generic-ev-falsify ,falsify)
-                   (glcp-generic-geval-name (lambda () ',geval))
-                   (glcp-generic-apply-concrete ,apply-concrete)
-                   (glcp-generic-apply-concrete-guard-wrapper ,apply-concrete))
+                   ;; (glcp-generic-apply-concrete ,apply-concrete)
+                   ;; (glcp-generic-apply-concrete-guard-wrapper ,apply-concrete)
+                   )
                   :in-theory (e/d** ((:ruleset ,f-i-lemmas)))
-                  :expand ((,interp-list x alist hyp clk obligs
-                                         overrides world state)
-                           (,interp-term x alist hyp clk obligs
-                                         overrides world state))
+                  :expand ((,interp-list x alist hyp clk obligs config state)
+                           (,interp-term x alist hyp clk obligs config state))
                   :do-not-induct t)
                  (and stable-under-simplificationp
                       '(:in-theory (e/d** ((:ruleset ,f-i-lemmas)
+                                           (:ruleset ,constraints)))))
+                 (and stable-under-simplificationp
+                      '(:in-theory (e/d** ((:ruleset ,f-i-lemmas)
+                                           (:ruleset ,constraints)
+                                           ;; ,apply-concrete
                                            ,(incat ev (symbol-name ev)
-                                                   "-CONSTRAINT-0")))))))
+                                                   "-OF-FNCALL-ARGS")))))))
 
         ;; Prove correctness of the clause processor.
         (defthm ,correct-thm
           (implies (and (pseudo-term-listp clause)
                         (alistp alist)
+                        (,meta-facts)
                         (,ev (conjoin-clauses
                               (acl2::clauses-result
                                (,clause-proc clause hints state)))
@@ -615,24 +476,28 @@
                         (glcp-generic-ev-lst ,ev-lst)
                         (glcp-generic-geval ,geval)
                         (glcp-generic-run-gified ,run-gified)
-                        (glcp-generic-run-gified-guard-wrapper ,run-gified)
-                        (glcp-generic-apply-concrete ,apply-concrete)
-                        (glcp-generic-apply-concrete-guard-wrapper ,apply-concrete)
+                        ;; (glcp-generic-apply-concrete ,apply-concrete)
+                        ;; (glcp-generic-apply-concrete-guard-wrapper ,apply-concrete)
                         (glcp-generic-ev-falsify ,falsify)
-                        (glcp-generic-geval-name (lambda ()
-                                                   ',geval))
+                        (glcp-generic-ev-meta-extract-global-badguy
+                         ,badguy)
                         (glcp-generic-run-parametrized
                          ,run-parametrized)
                         (glcp-generic-run-cases ,run-cases)
-                        (glcp-generic ,clause-proc)
-                        (glcp-generic-clause-proc-name
-                         (lambda () ',clause-proc))))
+                        (glcp-generic ,clause-proc)))
                   (case-match clause
                     ((('equal (fn . args) . &))
                      (and (member fn '(,clause-proc
                                       ,run-parametrized
                                       ,run-cases))
-                          `(:expand ((,fn . ,args)))))))
+                          `(:expand ((,fn . ,args)))))
+                    (((ev ('acl2::meta-extract-global-fact+ . &)
+                          . &)
+                      ('not (ev ('acl2::meta-extract-global-fact+ . &)
+                                . &)))
+                     '(:use ,badguy))
+                    ((('true-listp . &) . &)
+                     '(:use ,badguy))))
           :otf-flg t
           :rule-classes :clause-processor)
 
@@ -643,58 +508,63 @@
 
 
 (defmacro def-gl-clause-processor
-  (name apply-fns &key (output
-                        '(:off (warning warning! observation prove
-                                        event summary proof-tree
-                                        acl2::expansion)
-                               :gag-mode nil))
-        top-apply-fns
-        include-nonrec)
+  (name &rest rest-args
+        ;; apply-fns &key (output
+        ;;                 '(:off (warning warning! observation prove
+        ;;                                 event summary proof-tree
+        ;;                                 acl2::expansion)
+        ;;                        :gag-mode nil))
+        ;; top-apply-fns
+        ;; include-nonrec
+        )
   ":Doc-section ACL2::GL
-Define a GL clause processor with a given set of built-in functions.~/
+Define a GL clause processor with a given set of built-in symbolic counterparts.~/
 
 Usage:
 ~bv[]
  (def-gl-clause-processor my-gl-clause-processor
-   functions
-   ;; optional
-   :output with-output-settings
-   :top-apply-fns some-more-functions
-   :include-nonrec include-nonrecp)
+   :output with-output-settings)
 ~ev[]
-where ~c[some-functions] is a form whose value is a list of function
-symbols and with-output-settings is a list such as ~c[(:off :all)] of
-arguments approprate for ~il[with-output].  The other two arguments are
-discussed below.
-
 The above form defines a GL clause processor function named
-my-gl-clause-processor.  The list of function symbols provided and
-their descendants compose the set of ACL2 functions that the clause
-processor can execute directly on concrete values instead of
-interpreting.~/
+my-gl-clause-processor.  This clause processor is defined so that it can
+execute all existing symbolic counterpart functions.
 
-Each GL clause processor has two associated sets of functions: those
-it can directly execute symbolically, and those it can directly
-execute concretely.  DEF-GL-CLAUSE-PROCESSOR always uses the full set
-of functions with defined symbolic counterparts as the first set, but
-allows the user to provide the second set.  (Symbolic counterparts may
-be defined by hand or using ~c[MAKE-G-WORLD].)
+There is rarely a necessity for a user to define a new GL clause processor now,
+unless they have added symbolic counterpart functions either by hand-coding
+them or using ~c[MAKE-G-WORLD].~/
 
-The set of functions the clause processor will be able to execute
-concretely is determined by the non-optional ~c[functions] argument
-and the ~c[top-apply-fns] and ~c[include-nonrec] keyword arguments.
-By default, the new clause processor will be able to concretely
-execute the functions listed in the ~c[functions] argument and all of
-their recursive ancestors, in addition to the functions listed in the
-~c[top-apply-fns] argument and some functions that are always
-included.  However, if ~c[include-nonrec] is set to T, then all of
-these functions' ancestors are included, whether recursive or not.
+Each GL clause processor has an associated sets of functions that it can
+directly execute symbolically.  DEF-GL-CLAUSE-PROCESSOR makes a new processor
+that can execute the full set of existing symbolic counterparts.
+ (Symbolic counterparts may be defined by hand or using ~c[MAKE-G-WORLD].)
+
+It used to be the case that the clause processor also had a fixed set of
+functions it could execute concretely.  That is no longer the case.  We still
+accept the old form of def-gl-clause-processor, which takes an additional
+argument after the name of the clause processor and before the :output keyword
+ (if any).  However, this is deprecated and a message will be printed saying so.
 
 ~l[DEF-GL-THM] and ~il[GL-HINT] for information on using the GL
 clause processor to prove theorems.~/"
+  (b* (((mv oldp keys)
+        (if (keywordp (car rest-args))
+            (mv nil rest-args)
+          (mv t (cdr rest-args))))
+       (- (and oldp
+               (cw "DEPRECATED (def-gl-clause-proc): Def-gl-clause-proc now ~
+                    takes fewer arguments than it used to; in particular, the ~
+                    old APPLY-FNS argument is now ignored.  See :doc ~
+                    def-gl-clause-proc for the new syntax.~%")))
+       (output-look (assoc-keyword :output keys))
+       (output (if output-look
+                   (cadr output-look)
+                 '(:off (warning warning! observation prove
+                                 event summary proof-tree
+                                 acl2::expansion)
+                   :gag-mode nil))))
   `(make-event
     (def-gl-clause-processor-fn
-      ',name ,apply-fns ,include-nonrec ,top-apply-fns ',output state)))
+      ',name ',output state))))
 
 
 
@@ -754,7 +624,7 @@ clause processor to prove theorems.~/"
 
 
 
-(defmacro glcp-coverage-default-hint  (&key do-not-expand cov-theory-add)
+(defmacro glcp-coverage-default-hint  (&key do-not-expand cov-theory-add def-alist)
   `'(:computed-hint-replacement
      ((and stable-under-simplificationp
            (let ,'((last (car (last clause))))
@@ -767,6 +637,7 @@ clause processor to prove theorems.~/"
                       (e/d** ((:ruleset shape-spec-obj-in-range-open)))))
                    (acl2::structural-decomp-hint-fast
                     clause ',x stable-under-simplificationp state
+                    (append ,',def-alist (table-alist 'acl2::structural-decomp-defs (w state)))
                     (list* 'binary-and* 'booleanp ,',do-not-expand)))
                   :clause-processor (acl2::remove-irrel-cp clause ',x)))))))
      :in-theory
@@ -799,7 +670,7 @@ See :DOC GL::COVERAGE-PROOFS.
 
 
 (defun gl-hint-fn (clause-proc bindings param-bindings hyp param-hyp
-                               concl hyp-clk concl-clk param-clk
+                               concl hyp-clk concl-clk
                                cov-hints cov-hints-position cov-theory-add
                                do-not-expand hyp-hints result-hints
                                n-counterexamples abort-indeterminate
@@ -808,17 +679,37 @@ See :DOC GL::COVERAGE-PROOFS.
                                case-split-override case-split-hints test-side-goals)
   (declare (xargs :mode :program))
   `(b* (((mv clause-proc bindings param-bindings hyp param-hyp concl
-             hyp-clk concl-clk param-clk cov-hints cov-hints-position
+             hyp-clk concl-clk cov-hints cov-hints-position
              cov-theory-add do-not-expand hyp-hints result-hints
              n-counterexamples abort-indeterminate abort-ctrex exec-ctrex abort-vacuous
              run-before-cases run-after-cases case-split-override
              case-split-hints test-side-goals)
          (mv ',clause-proc ,bindings ,param-bindings ,hyp ,param-hyp
-             ,concl ,hyp-clk ,concl-clk ,param-clk ',cov-hints
+             ,concl ,hyp-clk ,concl-clk ',cov-hints
              ',cov-hints-position ',cov-theory-add ',do-not-expand
              ',hyp-hints ',result-hints ,n-counterexamples
              ,abort-indeterminate ,abort-ctrex ,exec-ctrex ,abort-vacuous ',run-before-cases ',run-after-cases
              ,case-split-override ',case-split-hints ,test-side-goals))
+
+        ;; ((er overrides)
+        ;;  (preferred-defs-to-overrides
+        ;;   (table-alist 'preferred-defs (w state)) state))
+
+        (config (make-glcp-config
+                 :abort-unknown abort-indeterminate
+                 :abort-ctrex abort-ctrex
+                 :exec-ctrex exec-ctrex
+                 :abort-vacuous abort-vacuous
+                 :nexamples n-counterexamples
+                 :hyp-clk hyp-clk
+                 :concl-clk concl-clk
+                 :clause-proc-name clause-proc
+                 ;; :overrides overrides
+                 :overrides nil ;; they get generated inside the clause proc
+                 :run-before (and (not test-side-goals) run-before-cases)
+                 :run-after (and (not test-side-goals) run-after-cases)
+                 :case-split-override case-split-override))
+
         (cov-hints (glcp-coverage-hints
                     do-not-expand cov-theory-add cov-hints cov-hints-position)) 
         ((er trhyp)
@@ -845,12 +736,7 @@ bindings:
         (param-bindings (add-param-var-bindings vars param-bindings))
         (call `(,(if test-side-goals 'glcp-side-goals-clause-proc clause-proc)
                 clause (list ',bindings ',param-bindings ',trhyp
-                             ',trparam ',trconcl ',concl ,hyp-clk ,concl-clk
-                             ,param-clk ,n-counterexamples
-                             ,abort-indeterminate ,abort-ctrex ,exec-ctrex ,abort-vacuous
-                             ',(and (not test-side-goals) run-before-cases)
-                             ',(and (not test-side-goals) run-after-cases)
-                             ,case-split-override)
+                             ',trparam ',trconcl ',concl ',config)
                 state)))
      (value (glcp-combine-hints call cov-hints hyp-hints result-hints case-split-hints))))
 
@@ -858,7 +744,6 @@ bindings:
                                bindings param-bindings
                                (hyp-clk '1000000)
                                (concl-clk '1000000)
-                               (param-clk '1000000)
                                cov-hints cov-hints-position
                                cov-theory-add do-not-expand
                                hyp-hints
@@ -937,8 +822,7 @@ descriptions of each keyword argument:
           ;; Ignored unless case-splitting:
           :param-hyp               nil
           :run-before-cases        nil
-          :run-after-cases         nil
-          :param-clk               1000000)
+          :run-after-cases         nil)
 ~ev[]
 
 The keyword arguments to ~c[GL-HINT] are similar to ones for the
@@ -947,7 +831,7 @@ documented there.
 ~/"
 
   (gl-hint-fn clause-proc bindings param-bindings hyp param-hyp concl
-              hyp-clk concl-clk param-clk cov-hints cov-hints-position
+              hyp-clk concl-clk cov-hints cov-hints-position
               cov-theory-add do-not-expand hyp-hints result-hints
               n-counterexamples abort-indeterminate abort-ctrex exec-ctrex abort-vacuous
               run-before-cases run-after-cases
@@ -1026,11 +910,11 @@ in DEF-GL-THM.~%"))
   (declare (xargs :mode :program))
   (if clause-procp
       `(without-waterfall-parallelism
-         ,(def-gl-thm-fn name clause-proc rest))
+        ,(def-gl-thm-fn name clause-proc rest))
     `(without-waterfall-parallelism
-       (make-event
-        (let ((clause-proc (latest-gl-clause-proc)))
-          (def-gl-thm-fn ',name clause-proc ',rest))))))
+      (make-event
+       (let ((clause-proc (latest-gl-clause-proc)))
+         (def-gl-thm-fn ',name clause-proc ',rest))))))
 
 
 
@@ -1201,7 +1085,7 @@ for the theorem produced, as in ~c[defthm]; the default is ~c[:rewrite].
   (b* (((list hyp hyp-p param-hyp param-hyp-p concl concl-p cov-bindings
               cov-bindings-p param-bindings param-bindings-p
               cov-hints cov-hints-position cov-theory-add do-not-expand
-              hyp-clk concl-clk param-clk n-counterexamples
+              hyp-clk concl-clk n-counterexamples
               abort-indeterminate abort-ctrex exec-ctrex abort-vacuous run-before-cases run-after-cases
               case-split-override case-split-hints test-side-goals rule-classes)
         rest)
@@ -1218,7 +1102,6 @@ PARAM-BINDINGS must be provided in DEF-GL-PARAM-THM.~%"))
                          :param-bindings ,param-bindings
                          :hyp-clk ,hyp-clk
                          :concl-clk ,concl-clk
-                         :param-clk ,param-clk
                          :cov-hints ,cov-hints
                          :cov-hints-position ,cov-hints-position
                          :cov-theory-add ,cov-theory-add
@@ -1279,7 +1162,6 @@ PARAM-BINDINGS must be provided in DEF-GL-PARAM-THM.~%"))
         do-not-expand 
         (hyp-clk '1000000)
         (concl-clk '1000000)
-        (param-clk '1000000)
         (n-counterexamples '3)
         (abort-indeterminate 't) (abort-ctrex 't) (exec-ctrex 't) (abort-vacuous 't)
         run-before-cases run-after-cases
@@ -1300,7 +1182,7 @@ Usage:
 
    :rule-classes <rule classes expression>
 
-   :hyp-clk <number> :concl-clk <number> :param-clk <number>
+   :hyp-clk <number> :concl-clk <number>
    :clause-proc <clause processor name>
 
    :n-counterexamples <number>
@@ -1403,11 +1285,6 @@ variable ~c[id], which is bound to the current assignment of the case-splitting
 variables.  These can be used to print a message before and after running each
 case so that the user can monitor the theorem's progress.
 
-~c[:PARAM-CLK] is similar to ~c[:HYP-CLK] and ~c[:CONCL-CLK]
- (~l[DEF-GL-THM]); it governs the recursion depth of the symbolic simulation
-showing that the disjunction of the case assumptions is complete.  Its default
-is 1000000.
-
 By default, if a counterexample is encountered on any of the cases, the proof
 will abort.  Setting ~c[:ABORT-CTREX] to ~c[NIL] causes it to go on; the proof
 will fail after the clause processor returns because it will produce a goal of
@@ -1416,14 +1293,14 @@ will fail after the clause processor returns because it will produce a goal of
 By default, if any case hypothesis is unsatisfiable, the proof will abort.
 Setting ~c[:ABORT-VACUOUS] to ~c[NIL] causes it to go on.
 
-~/   
+~/
 "
   (declare (ignore skip-g-proofs local))
   (def-gl-param-thm-find-cp name clause-proc clause-procp
     (list hyp hyp-p param-hyp param-hyp-p concl concl-p cov-bindings
           cov-bindings-p param-bindings param-bindings-p cov-hints
           cov-hints-position cov-theory-add do-not-expand hyp-clk concl-clk
-          param-clk n-counterexamples abort-indeterminate abort-ctrex exec-ctrex
+          n-counterexamples abort-indeterminate abort-ctrex exec-ctrex
           abort-vacuous run-before-cases run-after-cases case-split-override
           case-split-hints test-side-goals rule-classes)))
 
