@@ -3,6 +3,7 @@
 
 (include-book "bfr")
 (include-book "gobjectp")
+(include-book "gtypes")
 (include-book "tools/mv-nth" :dir :system)
 (include-book "tools/bstar" :dir :system)
 (include-book "tools/defevaluator-fast" :dir :system)
@@ -10,10 +11,8 @@
 (include-book "centaur/misc/evaluator-metatheorems" :dir :system)
 (include-book "centaur/misc/interp-function-lookup" :dir :system)
 (include-book "centaur/ubdds/witness" :dir :system)
-(local (include-book "unicode/take" :dir :system))
-(local (include-book "gobjectp-thms"))
-
-
+(local (include-book "std/lists/take" :dir :system))
+(local (include-book "gtype-thms"))
 
 (local (defun before-run-gified-ev-tag () nil))
 
@@ -36,15 +35,20 @@
    (acl2::kwote-lst lst)
    (symbolp x)
    (pairlis$ a b)
+   (cons a b)
+   (binary-+ a b)
    (hide a)
    (mv-nth n x)
    (mv-list n x)
    (acl2::return-last a b c)
    (force x)
+   (gobj-listp x)
    (bfr-eval x env)
-   (bfr-p x)
-   (gobjectp x)
-   (gobject-listp x)))
+   (acl2::typespec-check ts x)
+   (iff a b)
+   (binary-+ a b)
+   (unary-- a)
+   (len x)))
 
 (defchoose run-gified-ev-falsify (a) (x)
   (not (run-gified-ev x a)))
@@ -123,9 +127,9 @@
 
 (defun run-gified-lhs-and-okp-breakdown (lhs okp)
   (case-match okp
-    (('mv-nth ''0 (acl2::fn . '(fn actuals hyp clk)))
+    (('mv-nth ''0 (acl2::fn . '(fn actuals hyp clk state)))
      (case-match lhs
-       ((ev ('mv-nth ''1 (!fn . '(fn actuals hyp clk))) 'env)
+       ((ev ('mv-nth ''1 (!fn . '(fn actuals hyp clk state))) 'env)
         (mv nil ev acl2::fn))
        (& (mv "lhs mismatched" nil nil))))
     (& (mv "okp mismatched" nil nil))))
@@ -136,10 +140,10 @@
      (run-gified-lhs-and-okp-breakdown lhs okp)
      (implies (not erp)
               (and (equal lhs
-                          `(,geval (mv-nth '1 (,run-gified fn actuals hyp clk))
+                          `(,geval (mv-nth '1 (,run-gified fn actuals hyp clk state))
                                    env))
                    (equal okp
-                          `(mv-nth '0 (,run-gified fn actuals hyp clk))))))
+                          `(mv-nth '0 (,run-gified fn actuals hyp clk state))))))
    :rule-classes :forward-chaining))
 
 (in-theory (disable run-gified-lhs-and-okp-breakdown))
@@ -281,7 +285,7 @@
              (if (and (equal the-geval geval)
                       (evals-of-formalsp formals evals geval 'env)
                       (nonnil-symbol-listp gformals)
-                      (acl2::no-dupsp gformals)
+                      (acl2::fast-no-duplicatesp gformals)
                       (not (member 'env gformals))
                       (equal (nthcdr (- nformals 2) gformals) '(hyp clk)))
                  (mv nil gformals)
@@ -357,6 +361,14 @@
    (equal (nth n (nthcdr m y))
           (nth (+ (nfix n) (nfix m)) y))))
 
+;; (encapsulate nil
+;;   (local (include-book "arithmetic/top-with-meta" :dir :system))
+;;   (defthmd equal-nthcdr-cons
+;;     (equal (equal (nthcdr n x) (cons a b))
+;;            (and (equal (nth n x) a)
+;;                 (< (nfix n) (len x))
+;;                 (equal (nthcdr (+ 1 (nfix n)) x) b)))
+;;     :hints (("goal" :induct (nthcdr n x)))))
 
 
            
@@ -521,10 +533,10 @@
    :hints(("Goal" :in-theory (enable run-gified-ev-constraint-0)))))
 
 (local
- (defthm run-gified-ev-lst-simple-take
-   (equal (run-gified-ev-lst (acl2::simpler-take n x) a)
-          (acl2::simpler-take n (run-gified-ev-lst x a)))
-   :hints(("Goal" :in-theory (enable acl2::simpler-take)))))
+ (defthm run-gified-ev-lst-take
+   (equal (run-gified-ev-lst (acl2::take n x) a)
+          (acl2::take n (run-gified-ev-lst x a)))
+   :hints(("Goal" :in-theory (enable acl2::take-redefinition)))))
 
 (local
  (defthm list-fix-run-gified-ev-lst
@@ -560,8 +572,9 @@
                                   geval 'env))
                         a)))))
    :hints(("Goal"
-           :in-theory (disable nth-of-nthcdr assoc-equal-pairlis-nth
-                               run-gified-check-geval-thm)
+           :in-theory (e/d () ;; equal-nthcdr-cons
+                           (nth-of-nthcdr assoc-equal-pairlis-nth
+                                          run-gified-check-geval-thm))
            :use ((:instance run-gified-check-geval-thm-form)
                  (:instance run-gified-check-geval-thm-formals)
                  (:instance
@@ -754,8 +767,8 @@
 
 
 (local
- (defthm len-simpler-take
-   (Equal (len (acl2::simpler-take n lst)) (nfix n))))
+ (defthm len-take
+   (Equal (len (acl2::take n lst)) (nfix n))))
 
 
 (local
@@ -775,14 +788,14 @@
 
 
 (local
- (in-theory (disable gobjectp-def equal-of-booleans-rewrite
+ (in-theory (disable equal-of-booleans-rewrite
                      default-car default-cdr)))
 
 
 
-(defun geval-car-cdr-of-gobject-listp-thm (geval rune)
+(defun geval-car-cdr-of-gobj-listp-thm (geval rune)
   `((not (use-by-hint ',rune))
-    (not (gobject-listp x))
+    (not (gobj-listp x))
     (if (equal (car (,geval x env))
                (,geval (car x) env))
         (equal (cdr (,geval x env))
@@ -790,131 +803,132 @@
       'nil)))
 
 (local
- (defthm geval-car-cdr-of-gobject-listp-thm-correct
+ (defthm geval-car-cdr-of-gobj-listp-thm-correct
    (implies (and (run-gified-ev-theoremp
-                  (disjoin (geval-car-cdr-of-gobject-listp-thm
+                  (disjoin (geval-car-cdr-of-gobj-listp-thm
                             geval rune)))
                  ;;    (gobjectp (run-gified-ev x a))
                  ;;                 (or (atom (run-gified-ev x a))
                  ;;                     (gobjectp (car (run-gified-ev x a))))
-                 (gobject-listp (run-gified-ev x a))
+                 (gobj-listp (run-gified-ev x a))
                  (not (equal geval 'quote)))
             (and (equal (car (run-gified-ev (list geval x env) a))
                         (run-gified-ev (list geval `(car ,x) env) a))
                  (equal (cdr (run-gified-ev (list geval x env) a))
                         (run-gified-ev (list geval `(cdr ,x) env) a))))
-   :hints(("Goal" :in-theory (enable use-by-hint run-gified-ev-constraint-0)
+   :hints(("Goal" :in-theory (enable use-by-hint run-gified-ev-constraint-0
+                                     geval-car-cdr-of-gobj-listp-thm)
            :use ((:instance run-gified-ev-falsify
-                            (x (disjoin (geval-car-cdr-of-gobject-listp-thm
+                            (x (disjoin (geval-car-cdr-of-gobj-listp-thm
                                          geval rune)))
                             (a `((x . ,(run-gified-ev x a))
                                  (env . ,(run-gified-ev env a))))))))))
 
-;; (defthm gobject-listp-simple-take-implies
-;;   (implies (and (gobject-listp (acl2::simple-take n x))
+;; (defthm gobj-listp-simple-take-implies
+;;   (implies (and (gobj-listp (acl2::simple-take n x))
 ;;                 (natp n) (< 0 n))
 ;;            (or (atom x) (gobjectp (car x))))
 ;;   :rule-classes nil)
            
 
 
-;; (defthm geval-car-cdr-of-gobject-listp-thm-correct-corollary
+;; (defthm geval-car-cdr-of-gobj-listp-thm-correct-corollary
 ;;   (implies (and (run-gified-ev-theoremp
-;;                  (disjoin (geval-car-cdr-of-gobject-listp-thm
+;;                  (disjoin (geval-car-cdr-of-gobj-listp-thm
 ;;                            geval rune)))
 ;;                 (gobjectp (run-gified-ev x a))
-;;                 (gobject-listp (acl2::simple-take n (run-gified-ev x a)))
+;;                 (gobj-listp (acl2::simple-take n (run-gified-ev x a)))
 ;;                 (natp n) (< 0 n)
 ;;                 (not (equal geval 'quote)))
 ;;            (and (equal (car (run-gified-ev (list geval x env) a))
 ;;                        (run-gified-ev (list geval `(car ,x) env) a))
 ;;                 (equal (cdr (run-gified-ev (list geval x env) a))
 ;;                        (run-gified-ev (list geval `(cdr ,x) env) a))))
-;;   :hints(("Goal" :use (geval-car-cdr-of-gobject-listp-thm-correct
-;;                        (:instance gobject-listp-simple-take-implies
+;;   :hints(("Goal" :use (geval-car-cdr-of-gobj-listp-thm-correct
+;;                        (:instance gobj-listp-simple-take-implies
 ;;                                   (x (run-gified-ev x a))))
-;;           :in-theory (disable geval-car-cdr-of-gobject-listp-thm
-;;                               geval-car-cdr-of-gobject-listp-thm-correct)
+;;           :in-theory (disable geval-car-cdr-of-gobj-listp-thm
+;;                               geval-car-cdr-of-gobj-listp-thm-correct)
 ;;           :do-not-induct t))
 ;;   :rule-classes ((:rewrite :backchain-limit-lst (0 nil 0 nil nil nil))))
 
-;; (defthm geval-car-cdr-of-gobject-listp-thm-correct-corollary2
+;; (defthm geval-car-cdr-of-gobj-listp-thm-correct-corollary2
 ;;   (implies (and (run-gified-ev-theoremp
-;;                  (disjoin (geval-car-cdr-of-gobject-listp-thm
+;;                  (disjoin (geval-car-cdr-of-gobj-listp-thm
 ;;                            geval rune)))
-;;                 (gobject-listp (run-gified-ev x a))
+;;                 (gobj-listp (run-gified-ev x a))
 ;;                 (not (equal geval 'quote)))
 ;;            (and (equal (car (run-gified-ev (list geval x env) a))
 ;;                        (run-gified-ev (list geval `(car ,x) env) a))
 ;;                 (equal (cdr (run-gified-ev (list geval x env) a))
 ;;                        (run-gified-ev (list geval `(cdr ,x) env) a))))
-;;   :hints(("Goal" :use geval-car-cdr-of-gobject-listp-thm-correct
-;;           :in-theory (e/d (gobject-listp-impl-gobjectp)
-;;                           (geval-car-cdr-of-gobject-listp-thm
-;;                            geval-car-cdr-of-gobject-listp-thm-correct))
+;;   :hints(("Goal" :use geval-car-cdr-of-gobj-listp-thm-correct
+;;           :in-theory (e/d (gobj-listp-impl-gobjectp)
+;;                           (geval-car-cdr-of-gobj-listp-thm
+;;                            geval-car-cdr-of-gobj-listp-thm-correct))
 ;;           :do-not-induct t))
 ;;   :rule-classes ((:rewrite :backchain-limit-lst (0 1 nil))))
 
-(defun geval-consp-when-gobject-listp-thm (geval rune)
+(defun geval-consp-when-gobj-listp-thm (geval rune)
   `((not (use-by-hint ',rune))
     ;; (not (gobjectp x))
 ;;     (not (if (consp x)
 ;;              (gobjectp (car x))
 ;;            't))
-    (not (gobject-listp x))
+    (not (gobj-listp x))
     (equal (consp (,geval x env))
            (consp x))))
 
 (local
- (defthm geval-consp-when-gobject-listp-thm-correct
+ (defthm geval-consp-when-gobj-listp-thm-correct
    (implies (and (run-gified-ev-theoremp
-                  (disjoin (geval-consp-when-gobject-listp-thm
+                  (disjoin (geval-consp-when-gobj-listp-thm
                             geval rune)))
                  ;; (gobjectp (run-gified-ev x a))
                  ;;                 (or (atom (run-gified-ev x a))
                  ;;                     (gobjectp (car (run-gified-ev x a))))
-                 (gobject-listp (run-gified-ev x a))
+                 (gobj-listp (run-gified-ev x a))
                  (not (equal geval 'quote)))
             (equal (consp (run-gified-ev (list geval x env) a))
                    (consp (run-gified-ev x a))))
    :hints(("Goal" :in-theory (enable use-by-hint run-gified-ev-constraint-0)
            :use ((:instance run-gified-ev-falsify
-                            (x (disjoin (geval-consp-when-gobject-listp-thm
+                            (x (disjoin (geval-consp-when-gobj-listp-thm
                                          geval rune)))
                             (a `((x . ,(run-gified-ev x a))
                                  (env . ,(run-gified-ev env a))))))))))
 
-;; (defthm geval-consp-when-gobject-listp-thm-correct-corollary
+;; (defthm geval-consp-when-gobj-listp-thm-correct-corollary
 ;;   (implies (and (run-gified-ev-theoremp
-;;                  (disjoin (geval-consp-when-gobject-listp-thm
+;;                  (disjoin (geval-consp-when-gobj-listp-thm
 ;;                            geval rune)))
 ;;                 (gobjectp (run-gified-ev x a))
-;;                 (gobject-listp (acl2::simple-take n (run-gified-ev x a)))
+;;                 (gobj-listp (acl2::simple-take n (run-gified-ev x a)))
 ;;                 (natp n) (< 0 n)
 ;;                 (not (equal geval 'quote)))
 ;;            (equal (consp (run-gified-ev (list geval x env) a))
 ;;                   (consp (run-gified-ev x a))))
-;;   :hints(("Goal" :use (geval-consp-when-gobject-listp-thm-correct
-;;                        (:instance gobject-listp-simple-take-implies
+;;   :hints(("Goal" :use (geval-consp-when-gobj-listp-thm-correct
+;;                        (:instance gobj-listp-simple-take-implies
 ;;                                   (x (run-gified-ev x a))))
-;;           :in-theory (disable geval-consp-when-gobject-listp-thm-correct
-;;                               geval-consp-when-gobject-listp-thm)
+;;           :in-theory (disable geval-consp-when-gobj-listp-thm-correct
+;;                               geval-consp-when-gobj-listp-thm)
 ;;           :do-not-induct t))
 ;;   :rule-classes ((:rewrite :backchain-limit-lst (0 nil 0 nil nil nil))))
 
-;; (defthm geval-consp-when-gobject-listp-thm-correct-corollary2
+;; (defthm geval-consp-when-gobj-listp-thm-correct-corollary2
 ;;   (implies (and (run-gified-ev-theoremp
-;;                  (disjoin (geval-consp-when-gobject-listp-thm
+;;                  (disjoin (geval-consp-when-gobj-listp-thm
 ;;                            geval rune)))
-;;                 (gobject-listp (run-gified-ev x a))
+;;                 (gobj-listp (run-gified-ev x a))
 ;;                 (natp n) (< 0 n)
 ;;                 (not (equal geval 'quote)))
 ;;            (equal (consp (run-gified-ev (list geval x env) a))
 ;;                   (consp (run-gified-ev x a))))
-;;   :hints(("Goal" :use (geval-consp-when-gobject-listp-thm-correct)
-;;           :in-theory (e/d (gobject-listp-impl-gobjectp)
-;;                           (geval-consp-when-gobject-listp-thm-correct
-;;                            geval-consp-when-gobject-listp-thm))
+;;   :hints(("Goal" :use (geval-consp-when-gobj-listp-thm-correct)
+;;           :in-theory (e/d (gobj-listp-impl-gobjectp)
+;;                           (geval-consp-when-gobj-listp-thm-correct
+;;                            geval-consp-when-gobj-listp-thm))
 ;;           :do-not-induct t))
 ;;   :rule-classes ((:rewrite :backchain-limit-lst (0 0 nil nil nil))))
 
@@ -936,8 +950,8 @@
                             (a `((env . ,(run-gified-ev env a))))))))))
 
 
-(in-theory (disable geval-consp-when-gobject-listp-thm
-                    geval-car-cdr-of-gobject-listp-thm
+(in-theory (disable geval-consp-when-gobj-listp-thm
+                    geval-car-cdr-of-gobj-listp-thm
                     geval-of-nil-thm))
 
 (local
@@ -982,45 +996,46 @@
                                     nil)))
      :hints(("Goal" :in-theory (enable run-gified-ev-constraint-0))))
 
-   (defthm gobjectp-nth-gobject-listp
-     (implies (gobject-listp lst)
-              (gobjectp (nth n lst)))
-     :hints(("Goal" :in-theory (enable gobject-listp))))
+   ;; (defthm gobjectp-nth-gobj-listp
+   ;;   (implies (gobj-listp lst)
+   ;;            (gobjectp (nth n lst)))
+   ;;   :hints(("Goal" :in-theory (enable gobj-listp))))
 
-   (defthmd listp-nthcdr-gobject-listp
-     (implies (and (gobject-listp lst)
+   (defthmd listp-nthcdr-gobj-listp
+     (implies (and (gobj-listp lst)
                    (nthcdr n lst))
               (consp (nthcdr n lst)))
-     :hints(("Goal" :in-theory (enable gobject-listp))))
+     :hints(("Goal" :in-theory (enable gobj-listp))))
 
-   (defthm gobject-listp-nthcdr
-     (implies (gobject-listp lst)
-              (gobject-listp (nthcdr n lst)))
-     :hints(("Goal" :in-theory (enable gobject-listp))))
+   (defthm gobj-listp-nthcdr
+     (implies (gobj-listp lst)
+              (gobj-listp (nthcdr n lst)))
+     :hints(("Goal" :in-theory (enable gobj-listp))))
 
-   (defthm gobject-listp-simpler-take
-     (implies (gobject-listp gobj)
-              (gobject-listp (acl2::simpler-take n gobj)))
-     :hints(("Goal" :in-theory (enable gobject-listp acl2::simpler-take))))
+   (defthm gobj-listp-take
+     (implies (gobj-listp gobj)
+              (gobj-listp (acl2::take n gobj)))
+     :hints(("Goal" :in-theory (enable gobj-listp acl2::take-redefinition
+                                       acl2::repeat))))
 
    (defun count-down2-cdr (n m l)
      (if (zp n)
          (list m l)
        (count-down2-cdr (1- (nfix n)) (1- (nfix m)) (cdr l))))
 
-   (defthm gobject-listp-simpler-take1
-     (implies (and (gobject-listp (acl2::simpler-take m gobj))
+   (defthm gobj-listp-take1
+     (implies (and (gobj-listp (acl2::take m gobj))
                    (< (nfix n) (nfix m)))
-              (gobject-listp (acl2::simpler-take n gobj)))
+              (gobj-listp (acl2::take n gobj)))
      :hints (("goal" :induct (count-down2-cdr m n gobj)
-              :in-theory (enable gobject-listp acl2::simpler-take))))
+              :in-theory (enable gobj-listp acl2::take-redefinition))))
 
-   (Defthm gobjectp-nth-when-gobject-listp-simpler-take
-     (implies (and (gobject-listp (acl2::simpler-take m x))
-                   (< (nfix n) (nfix m)))
-              (gobjectp (nth n x)))
-     :hints (("goal" :induct (count-down2-cdr m n x)
-              :in-theory (enable gobject-listp))))
+   ;; (Defthm gobjectp-nth-when-gobj-listp-take
+   ;;   (implies (and (gobj-listp (acl2::take m x))
+   ;;                 (< (nfix n) (nfix m)))
+   ;;            (gobjectp (nth n x)))
+   ;;   :hints (("goal" :induct (count-down2-cdr m n x)
+   ;;            :in-theory (enable gobj-listp))))
 
    (defthm cheap-default-car
      (implies (not (consp x))
@@ -1035,14 +1050,14 @@
 
    (defthm nthcdr-run-gified-of-geval
      (implies (and (run-gified-ev-theoremp
-                    (disjoin (geval-consp-when-gobject-listp-thm
+                    (disjoin (geval-consp-when-gobj-listp-thm
                               gevalfn geval-consp)))
                    (run-gified-ev-theoremp
-                    (disjoin (geval-car-cdr-of-gobject-listp-thm
+                    (disjoin (geval-car-cdr-of-gobj-listp-thm
                               gevalfn geval-car-cdr)))
                    (not (equal gevalfn 'quote))
-                   (gobject-listp (run-gified-ev args a))
-                   ;; (acl2::simpler-take n (run-gified-ev args a))
+                   (gobj-listp (run-gified-ev args a))
+                   ;; (acl2::take n (run-gified-ev args a))
                    )
               (equal
                (nthcdr n (run-gified-ev (list gevalfn args env) a))
@@ -1052,13 +1067,13 @@
             (and stable-under-simplificationp
                  (cond
                   ((member-equal '(zp n) clause)
-                   '(:use ((:instance geval-car-cdr-of-gobject-listp-thm-correct
+                   '(:use ((:instance geval-car-cdr-of-gobj-listp-thm-correct
                                       (geval gevalfn)
                                       (x (list 'quote (run-gified-ev args a)))
                                       (env (list 'quote (run-gified-ev env a)))
                                       (a nil)
                                       (rune geval-car-cdr))
-                           (:instance geval-car-cdr-of-gobject-listp-thm-correct
+                           (:instance geval-car-cdr-of-gobj-listp-thm-correct
                                       (geval gevalfn)
                                       (x (list 'quote
                                                (nthcdr (+ -1 n)
@@ -1067,7 +1082,7 @@
                                       (a nil)
                                       (rune geval-car-cdr)))
                           :in-theory (e/d (my-run-gified-ev-constraint-0)
-                                          (geval-car-cdr-of-gobject-listp-thm-correct
+                                          (geval-car-cdr-of-gobj-listp-thm-correct
                                            nthcdr nth))))
                   ((member-equal '(not (zp n)) clause)
                    '(:in-theory (e/d (nthcdr) (nth))))))))
@@ -1075,13 +1090,13 @@
 
    (defthm nth-run-gified-of-geval
      (implies (and (run-gified-ev-theoremp
-                    (disjoin (geval-consp-when-gobject-listp-thm
+                    (disjoin (geval-consp-when-gobj-listp-thm
                               gevalfn geval-consp)))
                    (run-gified-ev-theoremp
-                    (disjoin (geval-car-cdr-of-gobject-listp-thm
+                    (disjoin (geval-car-cdr-of-gobj-listp-thm
                               gevalfn geval-car-cdr)))
                    (not (equal gevalfn 'quote))
-                   (gobject-listp (run-gified-ev args a)))
+                   (gobj-listp (run-gified-ev args a)))
               (equal
                (nth n (run-gified-ev (list gevalfn args env) a))
                (run-gified-ev (list gevalfn (list 'car (make-n-cdrs n args)) env) a)))
@@ -1108,8 +1123,6 @@
             (< (nfix n) (len x)))
      :hints(("Goal" :in-theory (enable equal-of-booleans-rewrite))))
 
-   (in-theory (disable gobjectp-cons-case gobjectp-def))
-
    (defthm kwote-lst-cons
      (equal (acl2::kwote-lst (cons a b))
             (cons (list 'quote a)
@@ -1139,9 +1152,9 @@
      (implies (natp n)
               (equal (run-gified-ev-lst (make-nths-matching-formals
                                          n formals actuals) a)
-                     (acl2::simpler-take (len formals)
+                     (acl2::take (len formals)
                                         (nthcdr n (run-gified-ev actuals a)))))
-     :hints(("Goal" :in-theory (enable nth nthcdr acl2::simpler-take))))
+     :hints(("Goal" :in-theory (enable nth nthcdr acl2::take-redefinition))))
 
 
 
@@ -1156,18 +1169,18 @@
                (run-gified-ev-theoremp
                 (disjoin (acl2::ev-lookup-var-clause evalfn var-name)))
                (run-gified-ev-theoremp
-                (disjoin (geval-consp-when-gobject-listp-thm
+                (disjoin (geval-consp-when-gobj-listp-thm
                           gevalfn geval-consp)))
                (run-gified-ev-theoremp
                 (disjoin (geval-of-nil-thm
                           gevalfn geval-nil)))
                (run-gified-ev-theoremp
-                (disjoin (geval-car-cdr-of-gobject-listp-thm
+                (disjoin (geval-car-cdr-of-gobj-listp-thm
                           gevalfn geval-car-cdr)))
                (not (equal gevalfn 'quote))
                (not (equal evalfn 'quote))
-               ;; (gobject-listp (acl2::simpler-take n actuals))
-               (gobject-listp actuals))
+               ;; (gobj-listp (acl2::take n actuals))
+               (gobj-listp actuals))
               (equal (run-gified-ev-lst
                       (acl2::ev-apply-arglist-on-result
                        n
@@ -1183,19 +1196,20 @@
                      (run-gified-ev-lst
                       (make-evals-of-formals
                        (acl2::kwote-lst
-                        (acl2::simpler-take
+                        (acl2::take
                          n
                          actuals))
                        gevalfn env)
                       nil)))
      :hints(("Goal" 
              :induct t
-             :in-theory (enable gobject-listp acl2::simpler-take)
+             :in-theory (enable gobj-listp acl2::take-redefinition)
              :expand ((:free (a b c)
                              (acl2::ev-apply-arglist-on-result
                               n a b c))))
             (and stable-under-simplificationp
-                 '(:in-theory (e/d (my-run-gified-ev-constraint-0))))))
+                 '(:in-theory (e/d (my-run-gified-ev-constraint-0
+                                    acl2::repeat))))))
 
    (encapsulate nil
      (local (defthm nthcdr-is-nil
@@ -1207,17 +1221,17 @@
        (implies (and (nths-matching-formalsp idx formals varname list)
                      (natp idx))
                 (equal (run-gified-ev-lst list a)
-                       (acl2::simpler-take
+                       (acl2::take
                         (len formals)
                         (nthcdr idx (run-gified-ev varname a)))))
        :hints(("Goal" :in-theory (enable my-equal-of-cons
-                                         acl2::simpler-take
+                                         acl2::take-redefinition
                                          nths-matching-formalsp))))
 
      (defthmd nths-matching-formalsp-make-nths-matching-formals-ev1
        (implies (nths-matching-formalsp 0 formals varname list)
                 (equal (run-gified-ev-lst list a)
-                       (acl2::simpler-take
+                       (acl2::take
                         (len formals)
                         (run-gified-ev varname a))))
        :hints(("Goal" :in-theory (enable nthcdr)
@@ -1322,8 +1336,8 @@
  (encapsulate nil
 
    (local
-    (in-theory (disable geval-consp-when-gobject-listp-thm-correct
-                        cheap-default-car cheap-default-cdr acl2::simpler-take
+    (in-theory (disable geval-consp-when-gobj-listp-thm-correct
+                        cheap-default-car cheap-default-cdr acl2::take-redefinition
                         ev-quote-clause-correct-for-run-gified-ev
                         ev-lookup-var-clause-correct-for-run-gified-ev
                         nth-when-len-smaller
@@ -1354,16 +1368,14 @@
                         (:REWRITE RUN-GIFIED-EV-CONSTRAINT-3)
                         (:REWRITE ACL2::SYMBOLP-ASSOC-EQUAL)
                         (:DEFINITION ACL2::LIST-FIX)
-                        (:REWRITE GEVAL-CAR-CDR-OF-GOBJECT-LISTP-THM-CORRECT)
+                        (:REWRITE GEVAL-CAR-CDR-OF-GOBJ-LISTP-THM-CORRECT)
                         (:DEFINITION SYMBOL-LISTP)
                         (:REWRITE CHEAP-DEFAULT-CDR)
                         (:TYPE-PRESCRIPTION SYMBOL-LISTP))))
    (local
     (in-theory (set-difference-theories
                 (current-theory :here)
-                (list (EV-CONSTRAINT-FOR RUN-GIFIED-EV GOBJECT-LISTP)
-                      (EV-CONSTRAINT-FOR RUN-GIFIED-EV GOBJECTP)
-                      (EV-CONSTRAINT-FOR RUN-GIFIED-EV BFR-P)
+                (list (EV-CONSTRAINT-FOR RUN-GIFIED-EV GOBJ-LISTP)
                       (EV-CONSTRAINT-FOR RUN-GIFIED-EV BFR-EVAL)
                       (EV-CONSTRAINT-FOR RUN-GIFIED-EV FORCE)
                       (EV-CONSTRAINT-FOR RUN-GIFIED-EV MV-LIST)
@@ -1396,10 +1408,10 @@
                      (run-gified-ev-theoremp
                       (disjoin (acl2::ev-lookup-var-clause evalfn var-name)))
                      (run-gified-ev-theoremp
-                      (disjoin (geval-consp-when-gobject-listp-thm
+                      (disjoin (geval-consp-when-gobj-listp-thm
                                 gevalfn geval-consp)))
                      (run-gified-ev-theoremp
-                      (disjoin (geval-car-cdr-of-gobject-listp-thm
+                      (disjoin (geval-car-cdr-of-gobj-listp-thm
                                 gevalfn geval-car-cdr)))
                      (run-gified-ev-theoremp
                       (disjoin (geval-of-nil-thm
@@ -1409,7 +1421,7 @@
                      (mv-nth 0 (run-gified-ev body a))
                      (bfr-eval (cdr (assoc-equal 'hyp a))
                                (cadr (assoc-equal 'env a)))
-                     (gobject-listp (cdr (assoc-equal 'actuals a))))
+                     (gobj-listp (cdr (assoc-equal 'actuals a))))
                 (equal (run-gified-ev 
                         `(,gevalfn (mv-nth '1 ,body) env)
                         a)
@@ -1435,10 +1447,10 @@
                                                     BODY))))))
                           (:instance
                            nths-matching-formalsp-make-nths-matching-formals-ev1
-                           (list (ACL2::SIMPLER-TAKE (+ -2
+                           (list (ACL2::TAKE (+ -2
                                                        (LEN (MV-NTH 3 (RUN-GIFIED-CASE-BREAKDOWN BODY))))
                                                     (MV-NTH 3 (RUN-GIFIED-CASE-BREAKDOWN BODY))))
-                           (formals (ACL2::SIMPLER-TAKE
+                           (formals (ACL2::TAKE
                                      (+ -2
                                         (LEN (MV-NTH 3 (RUN-GIFIED-CASE-BREAKDOWN BODY))))
                                      (MV-NTH
@@ -1463,10 +1475,10 @@
                    (run-gified-ev-theoremp
                     (disjoin (acl2::ev-lookup-var-clause evalfn var-name)))
                    (run-gified-ev-theoremp
-                    (disjoin (geval-consp-when-gobject-listp-thm
+                    (disjoin (geval-consp-when-gobj-listp-thm
                               gevalfn geval-consp)))
                    (run-gified-ev-theoremp
-                    (disjoin (geval-car-cdr-of-gobject-listp-thm
+                    (disjoin (geval-car-cdr-of-gobj-listp-thm
                               gevalfn geval-car-cdr)))
                    (run-gified-ev-theoremp
                     (disjoin (geval-of-nil-thm
@@ -1476,7 +1488,7 @@
                    (mv-nth 0 (run-gified-ev body a))
                    (bfr-eval (cdr (assoc-equal 'hyp a))
                              (cadr (assoc-equal 'env a)))
-                   (gobject-listp (cdr (assoc-equal 'actuals a))))
+                   (gobj-listp (cdr (assoc-equal 'actuals a))))
               (equal (run-gified-ev 
                       (list gevalfn
                             (list 'quote (mv-nth 1 (run-gified-ev body a)))
@@ -1504,7 +1516,7 @@
         (acl2::simple-one-way-unify-lst
          '((implies
             (if (bfr-eval hyp (car env))
-                (if (gobject-listp actuals)
+                (if (gobj-listp actuals)
                     okp-term
                   'nil)
               'nil)
@@ -1533,7 +1545,7 @@
        (world (w state))
        (body (acl2::body run-gified-fn nil world))
        (body-clause (function-def-clause run-gified-fn run-gified-fn
-                                         '(fn actuals hyp clk)
+                                         '(fn actuals hyp clk state)
                                          body))
        ((when (eq ev-fn 'quote))
         (mv "The eval function is QUOTE which is silly."
@@ -1552,29 +1564,14 @@
              ev-fn (cdr (hons-get :quote eval-rule-alist)))
             (acl2::ev-lookup-var-clause
              ev-fn (cdr (hons-get :lookup-var eval-rule-alist)))
-            (geval-consp-when-gobject-listp-thm geval-fn geval-consp)
-            (geval-car-cdr-of-gobject-listp-thm geval-fn geval-car-cdr)
+            (geval-consp-when-gobj-listp-thm geval-fn geval-consp)
+            (geval-car-cdr-of-gobj-listp-thm geval-fn geval-car-cdr)
             (geval-of-nil-thm geval-fn geval-nil)
             body-clause
             clauses))))
 
 (local
- (defun run-gified-ev-alist (x al)
-   (if (atom x)
-       nil
-     (cons (cons (caar x) (run-gified-ev (cdar x) al))
-           (run-gified-ev-alist (cdr x) al)))))
-
-(local
- (acl2::def-functional-instance
-  simple-one-way-unify-lst-usage-for-run-gified-ev
-  acl2::simple-one-way-unify-lst-usage
-  ((acl2::unify-ev run-gified-ev)
-   (acl2::unify-ev-lst run-gified-ev-lst)
-   (acl2::unify-ev-alist run-gified-ev-alist))
-  :hints ((and stable-under-simplificationp
-               '(:in-theory (enable run-gified-ev-constraint-0))))))
-
+ (acl2::def-unify run-gified-ev run-gified-ev-alist))
 
 (local
  (defthm assoc-equal-run-gified-ev-alist
@@ -1627,12 +1624,12 @@
      (implies (not erp)
               (and (equal (run-gified-ev lhs a)
                           (run-gified-ev 
-                           `(,geval (mv-nth '1 (,run-gified fn actuals hyp clk))
+                           `(,geval (mv-nth '1 (,run-gified fn actuals hyp clk state))
                                     env)
                            a))
                    (equal (run-gified-ev okp a)
                           (run-gified-ev
-                           `(mv-nth '0 (,run-gified fn actuals hyp clk))
+                           `(mv-nth '0 (,run-gified fn actuals hyp clk state))
                            a)))))
    :hints (("goal" :use run-gified-lhs-and-okp-breakdown-correct
             :in-theory (disable run-gified-lhs-and-okp-breakdown-correct)))))
@@ -1655,6 +1652,7 @@
 (local
  (in-theory (disable acl2::ev-collect-apply-lemmas body table-alist w)))
 
+(local (in-theory (disable run-gified-process-body assoc-equal)))
 
 (defthm run-gified-clause-proc-correct
   (implies (and (pseudo-term-listp clause)
@@ -1704,7 +1702,7 @@
                   '(:use ((:instance run-gified-ev-falsify
                                      (x (disjoin (function-def-clause
                                                   run-gified-fn run-gified-fn
-                                                  '(fn actuals hyp clk)
+                                                  '(fn actuals hyp clk state)
                                                   (body run-gified-fn nil (w
                                                                            state)))))
                                      (a a)))
@@ -1713,7 +1711,7 @@
                  (acl2::simple-generalize-cp
                   clause '(((MV-NTH '1 (ACL2::SIMPLE-ONE-WAY-UNIFY-LST
                                         '((IMPLIES (IF (BFR-EVAL HYP (CAR ENV))
-                                                       (IF (GOBJECT-LISTP ACTUALS)
+                                                       (IF (GOBJ-LISTP ACTUALS)
                                                            OKP-TERM 'NIL)
                                                        'NIL)
                                                    (EQUAL LHS-TERM RHS-TERM)))
