@@ -25,7 +25,7 @@
 (include-book "extra-operations")
 (include-book "misc/hons-help2" :dir :system)
 (include-book "tools/rulesets" :dir :system)
-(local (include-book "unicode/take" :dir :system))
+(local (include-book "std/lists/take" :dir :system))
 
 (local (include-book "arithmetic/top" :dir :system))
 
@@ -38,6 +38,18 @@
 (in-theory (enable eval-bdd eval-bdd-list ubddp ubdd-listp
                    q-compose q-compose-list))
 
+(make-event
+
+; Disabling waterfall parallelism because this book allegedly uses memoization
+; while performing its proofs.
+
+ (if (and (hons-enabledp state) 
+          (f-get-global 'parallel-execution-enabled state)) 
+     (er-progn (set-waterfall-parallelism nil)
+               (value '(value-triple nil)))
+   (value '(value-triple nil))))
+
+
 ;; -------------------------------------------------------------------
 ;; Function Definitions
 ;; -------------------------------------------------------------------
@@ -521,8 +533,8 @@
 (local
  (encapsulate nil
    (local (include-book "arithmetic/top" :dir :system))
-   (defthm simpler-take-bfix-list
-     (equal (simpler-take (len vars) (bfix-list vars))
+   (defthm take-bfix-list
+     (equal (take (len vars) (bfix-list vars))
             (bfix-list vars)))))
 
 (defthm bfix-list-bfix-list
@@ -535,7 +547,7 @@
                                  vars)
                   (bfix-list
                    (take n (nthcdr start vars)))))
-  :hints(("Goal" :in-theory (enable simpler-take))))
+  :hints(("Goal" :in-theory (enable take-redefinition))))
 
 (defthm ubdd-listp-make-list-ac
   (equal (ubdd-listp (make-list-ac n nil acc))
@@ -574,8 +586,8 @@
 
 
 (local
- (defthm bfix-list-simpler-take-len
-   (equal (bfix-list (simpler-take (len x) x))
+ (defthm bfix-list-list-fix
+   (equal (bfix-list (list-fix x))
           (bfix-list x))))
 
 ;; (defthm eval-q-param-inv
@@ -614,7 +626,7 @@
                                                 vars))
                   (bfix-list vars)))
   :hints (("goal" :induct (eqpib-ind x vars)
-           :in-theory (enable max-depth))))
+           :in-theory (e/d (max-depth)))))
 
 (defthm len-q-param-inv-upper-bound
   (<= (len (q-param-inv x nvars))
