@@ -27,7 +27,7 @@
 (include-book "filemap")
 (include-book "../util/cwtime")
 (include-book "../mlib/warnings")
-(include-book "centaur/misc/ls" :dir :system)
+(include-book "oslib/ls" :dir :system)
 (include-book "str/top" :dir :system)
 (local (include-book "../util/arithmetic"))
 (local (include-book "../util/osets"))
@@ -285,6 +285,11 @@ additional details about this decision.</p>")
     (declare (xargs :guard (vl-override-p x)))
     (mergesort (vl-override-requirement-names-aux (vl-override->requirements x))))
 
+  (defthm true-listp-of-vl-override-requirement-names
+    (true-listp (vl-override-requirement-names x))
+    :rule-classes :type-prescription
+    :hints(("Goal" :in-theory (enable vl-override-requirement-names))))
+
   (defthm string-listp-of-vl-override-requirement-names
     (implies (force (vl-override-p x))
              (string-listp (vl-override-requirement-names x)))
@@ -782,7 +787,7 @@ overrides database loaded.</p>"
           (mv t nil nil defines comment-map walist state))
 
          (path1 (car dirs))
-         ((mv err filenames state) (acl2::ls-files path1 state))
+         ((mv err filenames state) (oslib::ls-files path1))
          (filenames (if err
                         (er hard? 'vl-read-overrides-aux "Error listing ~x0.~%" (car dirs))
                       filenames))
@@ -934,9 +939,9 @@ within @('tokens').</p>"
       (mv successp (reverse prefix-rev) rest)))
 
   (defttag vl-optimize)
+  (never-memoize vl-match-through-endmodule-aux)
   (progn!
    (set-raw-mode t)
-   (setf (gethash 'vl-match-through-endmodule-aux ACL2::*never-profile-ht*) t)
    (defun vl-match-through-endmodule (tokens)
      (b* (((mv successp prefix-rev rest)
            (vl-match-through-endmodule-aux tokens nil)))
@@ -1302,16 +1307,15 @@ eventually to check the \"requirements\" for each override.</li>
       (mv walist x-prime used modtokens)))
 
   (defttag vl-optimize)
-  (progn!
-   (set-raw-mode t)
-   (setf (gethash 'vl-apply-overrides-aux ACL2::*never-profile-ht*) t)
-   (defun vl-apply-overrides (x db walist)
-     (b* (((when (atom db))
-           (mv walist x nil nil))
-          ((mv walist x-prime-rev used modtokens)
-           (vl-apply-overrides-aux x db walist nil nil nil))
-          (x-prime (nreverse x-prime-rev)))
-       (mv walist x-prime used modtokens))))
+  (never-memoize vl-apply-overrides-aux)
+  (progn! (set-raw-mode t)
+          (defun vl-apply-overrides (x db walist)
+            (b* (((when (atom db))
+                  (mv walist x nil nil))
+                 ((mv walist x-prime-rev used modtokens)
+                  (vl-apply-overrides-aux x db walist nil nil nil))
+                 (x-prime (nreverse x-prime-rev)))
+              (mv walist x-prime used modtokens))))
   (defttag nil)
 
   (local (in-theory (enable vl-apply-overrides)))
