@@ -26,7 +26,7 @@
 (local (include-book "../util/arithmetic"))
 
 ;; BOZO some of these are expensive; consider backchain limits.
-(local (in-theory (disable consp-under-iff-when-true-listp
+(local (in-theory (disable acl2::consp-under-iff-when-true-listp
                            member-equal-when-member-equal-of-cdr-under-iff
                            default-car
                            default-cdr)))
@@ -404,7 +404,7 @@
                            ;args-exist-when-binary-op
                            ;args-exist-when-ternary-op
                            VL-PARSE-DRIVE-STRENGTH-OR-CHARGE-STRENGTH-FORWARD
-                           acl2::subsetp-equal-atom)))
+                           acl2::subsetp-when-atom-left)))
 
 
 
