@@ -25,6 +25,16 @@
 (local (include-book "../util/arithmetic"))
 (local (include-book "../util/esim-lemmas"))
 
+(make-event
+
+; Disabling waterfall parallelism because this book allegedly uses memoization
+; while performing its proofs.  
+
+ (if (and (ACL2::hons-enabledp state) 
+          (f-get-global 'ACL2::parallel-execution-enabled state)) 
+     (er-progn (set-waterfall-parallelism nil)
+               (value '(value-triple nil)))
+   (value '(value-triple nil))))
 
 (defsection resolving-multiple-drivers
   :parents (e-conversion)
@@ -385,7 +395,7 @@ for N=1 (in which case it acts like an ordinary assignment).</p>"
                             (;NO-DUPLICATESP-EQUAL-WHEN-SAME-LENGTH-MERGESORT
                              ;; leaving this one enabled--- NO-DUPLICATESP-EQUAL-OF-APPEND
                              ACL2::NO-DUPLICATESP-EQUAL-APPEND-IFF
-                             NO-DUPLICATESP-EQUAL-OF-APPEND-OF-APPEND))))))
+                             acl2::NO-DUPLICATESP-EQUAL-OF-APPEND-OF-APPEND))))))
 
 
 
