@@ -68,7 +68,8 @@ vl-pgenstr->val).</p>"
     (implies (and (not (equal other prefix))
                   (force (stringp prefix))
                   (force (stringp other)))
-             (not (vl-pgenstr-p other (vl-pgenstr prefix n))))))
+             (not (vl-pgenstr-p other (vl-pgenstr prefix n))))
+    :hints(("Goal" :in-theory (enable list-equiv)))))
 
 
 
@@ -219,7 +220,8 @@ number."
 
   (local (defthm lemma3
            (implies (not (member-equal key (strip-cars alist)))
-                    (not (hons-assoc-equal key alist)))))
+                    (not (hons-assoc-equal key alist)))
+           :hints(("Goal" :in-theory (enable hons-assoc-equal)))))
 
   (defthm hons-assoc-equal-when-vl-prefix-map-correct-p-aux
     (implies (and (vl-prefix-map-correct-p-aux domain pmap names)
@@ -248,9 +250,9 @@ number."
              (vl-prefix-map-correct-p-aux x pmap names))
     :hints(("Goal" :induct (len x))))
 
-  (defcong set-equivp equal (vl-prefix-map-correct-p-aux x pmap names) 1
+  (defcong set-equiv equal (vl-prefix-map-correct-p-aux x pmap names) 1
     :hints(("goal"
-            :in-theory (enable set-equivp)
+            :in-theory (enable set-equiv)
             :cases ((vl-prefix-map-correct-p-aux x pmap names)))))
 
   (defthm vl-prefix-map-correct-p-aux-of-hons-shrink-alist
@@ -286,7 +288,7 @@ number."
                               (alistp pset)
                               (vl-string-keys-p pset))))
   (and (or names (not pmap))
-       (set-equivp (strip-cars pset) (strip-cars pmap))))
+       (set-equiv (strip-cars pset) (strip-cars pmap))))
 
 (defaggregate vl-namedb
   (names pmap pset)
@@ -418,9 +420,9 @@ vl-compatible-with-prefix-set-p) for details.</p>")
           :in-theory (disable vl-namedb-okp-when-vl-namedb-p)
           :use ((:instance vl-namedb-okp-when-vl-namedb-p)))))
 
-(defthm vl-namedb->pset-under-set-equivp
+(defthm vl-namedb->pset-under-set-equiv
   (implies (force (vl-namedb-p x))
-           (set-equivp (strip-cars (vl-namedb->pset x))
+           (set-equiv (strip-cars (vl-namedb->pset x))
                           (strip-cars (vl-namedb->pmap x))))
   :hints(("Goal"
           :in-theory (disable vl-namedb-okp-when-vl-namedb-p)
@@ -565,7 +567,8 @@ fresh-name db-prime)')."
 
   (local (defthm lemma2
            (implies (hons-assoc-equal prefix pmap)
-                    (member-equal prefix (strip-cars pmap)))))
+                    (member-equal prefix (strip-cars pmap)))
+           :hints(("Goal" :in-theory (enable hons-assoc-equal)))))
 
   (local (defthm lemma3
            (implies (and (vl-prefix-map-correct-p pmap names)
@@ -655,9 +658,9 @@ matches a current prefix.</p>"
                      (vl-unlike-any-prefix-p name x))
             :hints(("Goal" :induct (len x)))))
 
-   (defcong set-equivp equal (vl-unlike-any-prefix-p name prefixes) 2
+   (defcong set-equiv equal (vl-unlike-any-prefix-p name prefixes) 2
      :hints(("Goal"
-             :in-theory (e/d (set-equivp)
+             :in-theory (e/d (set-equiv)
                              (vl-unlike-any-prefix-p))
              :cases ((vl-unlike-any-prefix-p name prefixes))))))
 
@@ -673,7 +676,8 @@ matches a current prefix.</p>"
             (implies (and (hons-assoc-equal key pset)
                           (vl-unlike-any-prefix-p name (strip-cars pset)))
                      (equal (vl-pgenstr-highest key (cons name names))
-                            (vl-pgenstr-highest key names)))))
+                            (vl-pgenstr-highest key names)))
+            :hints(("Goal" :in-theory (enable hons-assoc-equal)))))
 
    (local (defthm lemma3
             ;; This might not be a terrible rule to have in general.
@@ -755,7 +759,7 @@ note is printed and @('fresh-name') looks like @('name_n') instead."
   (defthm stringp-of-vl-namedb-plain-name
     (implies (force (stringp name))
              (stringp (mv-nth 0 (vl-namedb-plain-name name db))))
-    :hints(("Goal" :in-theory (disable vl-namedb->pset-under-set-equivp)))
+    :hints(("Goal" :in-theory (disable vl-namedb->pset-under-set-equiv)))
     :rule-classes :type-prescription)
 
   (defthm vl-namedb-p-of-vl-namedb-plain-name
@@ -808,7 +812,8 @@ note is printed and @('fresh-name') looks like @('name_n') instead."
    (local (defthm crock
             (implies (alistp alist)
                      (iff (member-equal prefix (strip-cars alist))
-                          (hons-assoc-equal prefix alist)))))
+                          (hons-assoc-equal prefix alist)))
+            :hints(("Goal" :in-theory (enable hons-assoc-equal)))))
 
    (local (defthm lemma
             (IMPLIES
