@@ -52,6 +52,8 @@ use FindBin qw($RealBin);
 use Getopt::Long qw(:config bundling_override);
 
 (do "$RealBin/certlib.pl") or die ("Error loading $RealBin/certlib.pl:\n!: $!\n\@: $@\n");
+(do "$RealBin/paths.pl") or die ("Error loading $RealBin/paths.pl:\n!: $!\n\@: $@\n");
+
 
 # use lib "/usr/lib64/perl5/5.8.8/x86_64-linux-thread-multi/Devel";
 # use Devel::DProf;
@@ -155,7 +157,7 @@ if it were part of the current file.
  - ;; cert_param: (<paramname>[=<paramval>])
      Cert_param directives control various things about how the file
 gets certified, such as whether it uses provisional certification
-(no_pcert), acl2x expansion (acl2x), and skip-proofs during acl2x
+(pcert), acl2x expansion (acl2x), and skip-proofs during acl2x
 expansion (acl2xskip).
 
 See files make-targets and regression-targets for example uses of
@@ -342,13 +344,25 @@ OPTIONS:
 
    --params <filename>
           Specifies a file that contains lines like:
-             mybook.cert:  no_pcert = 1, acl2x = 0
+             mybook.cert:  pcert = 1, acl2x = 0
           with the grammar:
              <bookname>.cert: <param> = <val> [ , <param> = <val> ]*
           The parameters specified have the same meaning as cert_param
           directives in the books themselves, and if the same parameter
           is assigned in the book and in the params file, the params
           file takes precedence.
+
+   --cache <filename>
+          Specifies a file to use as a sourcefile events cache.  Such
+          a file records events such as include-book, cert-param,
+          etc., that are present in each source file.  When this
+          option is given, cached information is read from that file
+          if it is present, and updated information is written to it
+          when we are done.  Using a cache may improve performance by
+          reducing the number of source files that must be scanned for
+          events.  File modification times are used to determine when
+          the cached information about a file must be updated.
+
 ';
 
 GetOptions ("help|h"               => sub { print $summary_str;
@@ -415,8 +429,7 @@ GetOptions ("help|h"               => sub { print $summary_str;
 					    read_targets(shift, \@user_targets);
 					},
 	    "debug"                => \$certlib_opts{"debugging"},
-	    # Note: this doesn't do anything yet.
-	    "cache|h=s"            => \$cache_file,
+	    "cache=s"              => \$cache_file,
 	    "accept-cache"         => \$certlib_opts{"believe_cache"},
 	    "deps-of|p=s"          => sub { shift; push(@user_targets, "-p " . shift); },
 	    "params=s"             => \$params_file,
@@ -441,9 +454,16 @@ unless ($acl2) {
     $acl2 = "acl2";
 }
 
-$acl2 = which($acl2);
+# convert user-provided ACL2 to cygwin path, under cygwin
+$acl2 = path_import($acl2);
+# this is probably always /dev/null but who knows under windows
+my $devnull = File::Spec->devnull;
+# get the absolute path
+$acl2 = `which $acl2 2>$devnull`;
+chomp($acl2);  # remove trailing newline
 
 if ($acl2) {
+    # canonicalize the path
     $acl2 = abs_canonical_path($acl2);
     unless($quiet || $no_build) {
 	print "ACL2 executable is ${acl2}\n";
@@ -459,11 +479,19 @@ flag or ACL2 environment variable.\n";
 
 if (! $acl2_books) {
     if ($acl2) {
-	$acl2_books = rel_path(dirname($acl2), "books");
+	# was:
+	# my $tmp_acl2_books = rel_path(dirname($acl2), "books");
+	my $tmp_acl2_books = File::Spec->catfile(dirname($acl2), "books");
+	if (-d $tmp_acl2_books) {
+	    $acl2_books = $tmp_acl2_books;
+	} else {
+	    $acl2_books = $RealBin;
+	}
     } else {
 	$acl2_books = $RealBin;
     }
 }
+
 $acl2_books = abs_canonical_path($acl2_books);
 unless($quiet) {
     print "System books directory is ${acl2_books}\n";
@@ -472,8 +500,10 @@ unless($quiet) {
 certlib_add_dir("SYSTEM", $acl2_books);
 # In case we're going to run Make, set the ACL2_SYSTEM_BOOKS
 # and ACL2 environment variables to match our assumption.
-$ENV{"ACL2_SYSTEM_BOOKS"} = $acl2_books;
-
+# In cygwin, ACL2 reads paths like c:/foo/bar whereas we're dealing in
+# paths like /cygdrive/c/foo/bar, so "export" it
+my $acl2_books_env = path_export($acl2_books);
+$ENV{"ACL2_SYSTEM_BOOKS"} = $acl2_books_env;
 
 my %tscache = ();
 
@@ -529,6 +559,40 @@ foreach my $run (@run_sources) {
     }
 }
 
+my @certs = sort(keys %depmap);
+
+# Warn about books that include relocation stubs
+my %stubs = (); # maps stub files to the books that include them
+foreach my $cert (@certs) {
+    my $certdeps = cert_bookdeps($cert, \%depmap);
+    foreach my $dep (@$certdeps) {
+	if (cert_get_param($dep, \%depmap, "reloc_stub")) {
+	    if (exists $stubs{$dep}) {
+		push(@{$stubs{$dep}}, $cert);
+	    } else {
+		$stubs{$dep} = [ $cert ];
+	    }
+	}
+    }
+}
+if (%stubs) {
+    print "Relocation warnings:\n";
+    print "--------------------------\n";
+    my @stubbooks = sort(keys %stubs);
+    foreach my $stub (@stubbooks) {
+	print "Stub file:       $stub\n";
+	# note: assumes each stub file includes only one book.
+	print "relocated to:    ${cert_bookdeps($stub, \%depmap)}[0]\n";
+	print "is included by:\n";
+	foreach my $cert (sort(@{$stubs{$stub}})) {
+	    print "                 $cert\n";
+	}
+	print "--------------------------\n";
+    }
+}
+
+
+
 my $mf_intro_string = '
 # Cert.pl is a build system for ACL2 books.  The cert.pl executable is
 # located under your ACL2_SYSTEM_BOOKS directory; run "cert.pl -h" for
@@ -568,9 +632,6 @@ unless ($no_makefile) {
     # declare $var_prefix_CERTS to be the list of certificates
     print $mf $var_prefix . "_CERTS :=";
 
-    my @certs = keys %depmap;
-    @certs = sort(@certs);
-
     foreach my $cert (@certs) {
 	print $mf " \\\n     $cert";
 	# if (cert_get_param($cert, \%depmap, "acl2x")) {
@@ -602,12 +663,26 @@ unless ($no_makefile) {
     }
     print $mf "\n\n";
 
+    # Write out the list of hons-only certs
+    # Propagate the hons-only requirement:
+    my %visited = ();
+    foreach my $cert (@certs) {
+	propagate_reqparam($cert, "hons-only", \%visited, \%depmap);
+    }
+    print $mf "${var_prefix}_HONS_ONLY :=";
+    foreach my $cert (@certs) {
+	if (cert_get_param($cert, \%depmap, "hons-only")) {
+	    print $mf " \\\n     $cert ";
+	}
+    }
+    print $mf "\n\n";
+
     # If there are labels, write out the sources and certs for those
     foreach my $label (sort(keys %labels)) {
 	my @topcerts = @{$labels{$label}};
 	my @labelcerts = ();
 	my @labelsources = ();
-	my %visited = ();
+	%visited = ();
 	# print "Processing label: $label\n";
 	foreach my $topcert (@topcerts) {
 	    # print "Visiting $topcert\n";
@@ -651,10 +726,10 @@ unless ($no_makefile) {
 	my $image = cert_image($cert, \%depmap);
 	my $useacl2x = cert_get_param($cert, \%depmap, "acl2x") || 0;
 	# BOZO acl2x implies no pcert
-	my $nopcert = $useacl2x || cert_get_param($cert, \%depmap, "no_pcert") || 0;
+	my $pcert_ok = ( ! $useacl2x && cert_get_param($cert, \%depmap, "pcert")) || 0;
 	my $acl2xskip = cert_get_param($cert, \%depmap, "acl2xskip") || 0;
 	print $mf "$cert : acl2x = $useacl2x\n";
-	print $mf "$cert : no_pcert = $nopcert\n";
+	print $mf "$cert : pcert = $pcert_ok\n";
 	# print $mf "#$cert params: ";
 	# my $params = cert_get_params($cert, \%depmap);
 	# while (my ($key, $val) = each %$params) {
@@ -667,7 +742,9 @@ unless ($no_makefile) {
 	}
 	if ($image && ($image ne "acl2")) {
 	    if ($bin_dir) {
-		print $mf " \\\n     " . rel_path($bin_dir, $image);
+		# was:
+		# print $mf " \\\n     " . rel_path($bin_dir, $image);
+		print $mf " \\\n     " . File::Spec->catfile($bin_dir, $image);
 	    } elsif (! $warned_bindir) {
 		print "Warning: no --bin set, so not adding image dependencies,\n";
 		print " e.g.   $cert : $image\n";
@@ -693,7 +770,9 @@ unless ($no_makefile) {
 	    }
 	    if ($image && ($image ne "acl2")) {
 		if ($bin_dir) {
-		    print $mf " \\\n     " . rel_path($bin_dir, $image);
+		    # was:
+		    # print $mf " \\\n     " . rel_path($bin_dir, $image);
+		    print $mf " \\\n     " . File::Spec->catfile($bin_dir, $image);
 		} elsif (! $warned_bindir) {
 		    print "Warning: no --bin set, so not adding image dependencies,\n";
 		    print " e.g.   $cert : $image\n";
@@ -711,7 +790,7 @@ unless ($no_makefile) {
     foreach my $cert (@certs) {
 	my $useacl2x = cert_get_param($cert, \%depmap, "acl2x") || 0;
 	# BOZO acl2x implies no pcert
-	my $nopcert = $useacl2x || cert_get_param($cert, \%depmap, "no_pcert") || 0;
+	my $pcert_ok = (! $useacl2x && cert_get_param($cert, \%depmap, "pcert")) || 0;
 	
 	my $bookdeps = cert_bookdeps($cert, \%depmap);
 	my $portdeps = cert_portdeps($cert, \%depmap);
@@ -719,14 +798,14 @@ unless ($no_makefile) {
 	my $otherdeps = cert_otherdeps($cert, \%depmap);
 	my $image = cert_image($cert, \%depmap);
 	(my $base = $cert) =~ s/\.cert$//;
-	# this is either the pcert0 or pcert1 depending on no_pcert
+	# this is either the pcert0 or pcert1 depending on pcert_ok
 	my $pcert = cert_sequential_dep($cert, \%depmap);
-	print $mf "$pcert : no_pcert = $nopcert\n";
+	print $mf "$pcert : pcert = $pcert_ok\n";
 	print $mf "$pcert : acl2x = $useacl2x\n";
 	print $mf "$pcert :";
 	foreach my $dep (@$bookdeps, @$portdeps) {
 	    # this is either the pcert0 or pcert1 depending whether the dependency
-	    # has no_pcert set
+	    # has pcert set
 	    print $mf " \\\n     " . cert_sequential_dep($dep, \%depmap);
 	}
 	foreach my $dep (@$srcdeps, @$otherdeps) {
@@ -734,7 +813,9 @@ unless ($no_makefile) {
 	}
 	if ($image && ($image ne "acl2")) {
 	    if ($bin_dir) {
-		print $mf " \\\n     " . rel_path($bin_dir, $image);
+		# was:
+		# print $mf " \\\n     " . rel_path($bin_dir, $image);
+		print $mf " \\\n     " . File::Spec->catfile($bin_dir, $image);
 	    } elsif (! $warned_bindir) {
 		print "Warning: no --bin set, so not adding image dependencies,\n";
 		print " e.g.   $cert : $image\n";
@@ -742,19 +823,18 @@ unless ($no_makefile) {
 	    }
 	}
 	print $mf "\n";
-	# If we're doing prov cert, pcert1 depends on pcert0
-	if (! $nopcert) {
+	# If we're doing prov cert, pcert1 depends on pcert0 and cert depends on pcert1
+	if ($pcert_ok) {
 	    # Pcert1 files depend only on the corresp. pcert0.
 	    print $mf "$base.pcert1 : acl2x = $useacl2x\n";
-	    print $mf "$base.pcert1 : no_pcert = $nopcert\n";
+	    print $mf "$base.pcert1 : pcert = $pcert_ok\n";
 	    print $mf "$base.pcert1 : $base.pcert0\n";
 	} elsif ($useacl2x) {
 	    # pcert1 depends on .acl2x
 	    print $mf "$base.pcert1 : $base.acl2x\n";
 	}
-	# Cert files depend on their certificate dependencies (above), and
-	# (order-only) on the corresponding .pcert1.
-	print $mf "$cert : | $base.pcert1\n\n";
+	print $mf "$cert : $base.pcert1\n";
+	print $mf "\n";
     }
 
     print $mf "\nendif\n\n";
