@@ -26,8 +26,9 @@ use strict;
 use warnings;
 use File::Basename;
 use File::Spec;
-use Storable;
-# use Cwd;
+# don't know how to get it on msys
+# use File::Which qw(which where);
+use Storable qw(nstore retrieve);
 use Cwd 'abs_path';
 
 my $cache_version_code = 3;
@@ -63,43 +64,46 @@ sub human_time {
 }
 
 
-
-
-sub rel_path {
-# Composes two paths together.  Basically just builds "$base/$path"
-# but handles the special cases where $base is empty or $path is
-# absolute.
-    my $base = shift;
+my %abs_path_memo = ();
+# must only be called on existing paths, at least in cygwin
+sub get_abs_path {
     my $path = shift;
-    if (substr($path,0,1) eq "/" || !$base) {
-	return $path;
-    } else {
-	return "$base/$path";
+    my $entry = $abs_path_memo{$path};
+    if ($entry) {
+	return $entry;
     }
+    my $res = Cwd::abs_path($path);
+    $abs_path_memo{$path}=$res;
+    return $res;
 }
 
-sub rec_readlink {
-    my $path = shift;
-    while (-l $path) {
-	$path = readlink $path;
-    }
-    return $path;
-}
-
-
+# We're mostly interested in resolving directory symlinks here.
+# If we have source files themselves symlinked, we won't canonicalize them
+# b/c presumably they are supposed to be considered different files?
+# This assumes that the file isn't necessarily supposed to exist, but the directory is.
 sub abs_canonical_path {
     my $path = shift;
-    my $abspath = File::Spec->rel2abs(rec_readlink($path));
+    # print "path: $path\n";
+    my $abspath = File::Spec->rel2abs($path);
+    # print "abspath: $abspath\n";
     my ($vol, $dir, $file) = File::Spec->splitpath($abspath);
-    if (! -d $dir) {
-	print "Oops, trying to go into $dir\n";
+    # print "path: $path vol: $vol dir: $dir file: $file\n";
+    my $voldir = File::Spec->catpath($vol, $dir, "");
+    # print "voldir: $voldir\n";
+    if (! -d $voldir) {
+	print "Oops, trying to go into $voldir\n";
 	return 0;
     }
-    my $absdir = Cwd::fast_abs_path($dir);
+    # fast_abs_path is supposed to be faster, but it seems not to be
+    # on a test system with linux over nfs etc etc.  Who knows.  Doc
+    # also says fast_abs_path is "more dangerous", whatever that
+    # means.
+    my $absdir = get_abs_path($voldir); # Cwd::fast_abs_path($voldir);
+    # print "absdir: $absdir\n";
     if ($absdir) {
-	return File::Spec->catpath($vol, $absdir, $file);
+	return File::Spec->catfile($absdir, $file);
     } else {
-	print "Warning: canonical_path: Directory not found: " . $dir . "\n";
+	print "Warning: canonical_path: Directory not found: " . $voldir . "\n";
 	return 0;
     }
 }
@@ -108,43 +112,29 @@ my $BASE_PATH = abs_canonical_path(".");
 
 my %canonical_path_memo = ();
 
+sub canonical_path_aux {
+    my $fname = shift;
+    
+    my $abs_path = abs_canonical_path($fname);
+    if ($BASE_PATH && $abs_path) {
+	my $relpath =  File::Spec->abs2rel($abs_path, $BASE_PATH);
+	return $relpath ? $relpath : ".";
+    }
+    return $abs_path;
+}
+
 sub canonical_path {
     my $fname = shift;
     my $entry = $canonical_path_memo{$fname};
     if ($entry) {
 	return $entry;
     } else {
-	my $res;
-	my $abs_path = abs_canonical_path($fname);
-	if ($BASE_PATH && $abs_path) {
-	    my $relpath =  File::Spec->abs2rel($abs_path, $BASE_PATH);
-	    $res = $relpath ? $relpath : ".";
-	} else {
-	    $res = $abs_path;
-	}
+	my $res = canonical_path_aux($fname);
 	$canonical_path_memo{$fname} = $res;
 	return $res;
     }
 }
 
-sub which {
-    my $name = shift;
-    # test to see if this is just a filename with no directory -- is this correct?
-    if (basename($name) eq $name) {
-	foreach my $dir (split(":", $ENV{"PATH"})) {
-	    my $file = rel_path($dir, $name);
-	    if ((! -d $file) && -x $file) {
-		return abs_canonical_path($file);
-	    }
-	}
-    } elsif ((! -d $name) && -x $name) {
-	return abs_canonical_path($name);
-    }
-    return 0;
-}
-	
-
-
 
 
 sub short_cert_name {
@@ -316,7 +306,7 @@ sub cert_sequential_dep {
     my ($certfile, $deps) = @_;
     my $res;
     if (cert_get_param($certfile, $deps, "acl2x")
-	|| cert_get_param($certfile, $deps, "no_pcert")) {
+	|| ! cert_get_param($certfile, $deps, "pcert")) {
 	($res = $certfile) =~ s/\.cert$/\.pcert1/;
     } else {
 	($res = $certfile) =~ s/\.cert$/\.pcert0/;
@@ -333,7 +323,7 @@ sub read_costs {
 	    my $pcert1file = cert_to_pcert1($certfile);
 	    $basecosts->{$certfile} = get_cert_time($certfile, $warnings, $use_realtime, $pcert);
 	    $basecosts->{$pcert1file} = get_cert_time($pcert1file, $warnings, $use_realtime, $pcert);
-	    if (! cert_get_param($certfile, $deps, "no_pcert")
+	    if (cert_get_param($certfile, $deps, "pcert")
 		&& ! cert_get_param($certfile, $deps, "acl2x")) {
 		# print "file: $certfile no_pcert: " . cert_get_param($certfile, $deps, "no_pcert") . "\n";
 		my $pcert0file = cert_to_pcert0($certfile);
@@ -406,7 +396,7 @@ sub compute_cost_paths_aux {
 	    }
 	} elsif ($target =~ /\.pcert1$/) {
 	    (my $certfile = $target) =~ s/\.pcert1$/\.cert/;
-	    if (cert_get_param($certfile, $deps, "no_pcert") ||
+	    if (! cert_get_param($certfile, $deps, "pcert") ||
 		cert_get_param($certfile, $deps, "acl2x")) {
 		## Depends on the sequential deps of the other certs
 		my $certdeps = cert_deps($certfile, $deps);
@@ -714,7 +704,7 @@ sub parallelism_stats {
     if ($curr_parallel != 0) {
 	print "Error: Ended with jobs still running??\n"
     }
-    my $avg_parallel = $running_total / $lasttime;
+    my $avg_parallel = ($lasttime != 0) ? $running_total / $lasttime : "???";
 
     return ($max_parallel, $max_start_time, $max_end_time, $avg_parallel, $running_total);
 }
@@ -869,12 +859,18 @@ sub get_loads {
 
 my $two_pass_warning_printed = 0;
 
+# Cert_param lines are currently of the form:
+# cert_param: ( foo = bar , baz = 1 , bla )
+# (the whitespace is optional.)
+# An entry without an = is just set to 1.
 sub parse_params {
     my $param_str = shift;
     my @params = split(/,/, $param_str);
     my @pairs = ();
     foreach my $param (@params) {
-	my @assign = $param =~ m/[\s]*([^\s=]*)[\s]*=[\s]*([^\s=]*)[\s]*/;
+	$param =~ s/^\s+//;
+	$param =~ s/\s+$//; #remove leading/trailing whitespace
+	my @assign = $param =~ m/([^\s=]*)[\s]*=[\s]*([^\s=]*)/;
 	if (@assign) {
 	    push(@pairs, [$assign[0], $assign[1]]);
 	} else {
@@ -889,7 +885,7 @@ sub parse_params {
 sub get_cert_param {
     my ($base,$the_line,$events) = @_;
 
-    my $regexp = "cert_param:[\\s]*\\(?([^)]*)\\)?";
+    my $regexp = "cert[-_]param:[\\s]*\\(?([^)]*)\\)?";
     my @match = $the_line =~ m/$regexp/;
     if (@match) {
 	debug_print_event($base, "cert_param", \@match);
@@ -914,6 +910,11 @@ sub get_cert_param {
 	push (@$events, [$cert_param_event, "acl2x", 1]);
 	return 1;
     }
+    $regexp = "\\(check-hons-enabled[\\s]+\\(:book";
+    if ($the_line =~ m/$regexp/) {
+	push (@$events, [$cert_param_event, "hons-only", 1]);
+	return 1;
+    }
     return 0;
 }
 
@@ -957,7 +958,8 @@ sub excludep {
     my $prev = shift;
     my $dirname = dirname($prev);
     while ($dirname ne $prev) {
-	if (-e rel_path($dirname, "cert_pl_exclude")) {
+	if (-e File::Spec->catfile($dirname, "cert_pl_exclude")) {
+	    # (-e rel_path($dirname, "cert_pl_exclude")) {
 	    return 1;
 	}
 	$prev = $dirname;
@@ -1050,13 +1052,17 @@ sub expand_dirname_cmd {
 	    return 0;
 	}
 	print "expand $dirname -> $dirpath\n" if $debugging;
-	$fullname = canonical_path(rel_path($dirpath, $relname . $ext));
+	# was:
+	# $fullname = canonical_path(rel_path($dirpath, $relname . $ext));
+	$fullname = canonical_path(File::Spec->catfile($dirpath, $relname . $ext));
 	if (! $fullname) {
 	    print ":dir entry in ($cmd \"$relname\" :dir $dirname) produced bad path\n";
 	}
     } else {
 	my $dir = dirname($basename);
-	$fullname = canonical_path(rel_path($dir, $relname . $ext));
+	# was:
+	# $fullname = canonical_path(rel_path($dir, $relname . $ext));
+	$fullname = canonical_path(File::Spec->catfile($dir, $relname . $ext));
 	if (! $fullname) {
 	    print "bad path in ($cmd \"$relname\")\n";
 	}
@@ -1132,7 +1138,9 @@ sub src_deps {
 	    my $name = $event->[1];
 	    my $dir = $event->[2];
 	    my $basedir = dirname($fname);
-	    my $newdir = canonical_path(rel_path($basedir, $dir));
+	    # was:
+	    # my $newdir = canonical_path(rel_path($basedir, $dir));
+	    my $newdir = canonical_path(File::Spec->catfile($basedir, $dir));
 	    if (! $newdir) {
 		print "Bad path processing (add-include-book-dir :$name \"$dir\") in $fname\n";
 	    }
@@ -1273,7 +1281,9 @@ sub find_deps {
 	( $base = $lispfile ) =~ s/\.lisp$//;
 	my $acl2file = $base . ".acl2";
 	if (! -e $acl2file) {
-	    $acl2file = rel_path(dirname($base), "cert.acl2");
+	    # was:
+	    # $acl2file = rel_path(dirname($base), "cert.acl2");
+	    $acl2file = File::Spec->catfile(dirname($base), "cert.acl2");
 	    if (! -e $acl2file) {
 		$acl2file = 0;
 	    }
@@ -1319,7 +1329,9 @@ sub find_deps {
 	# ACL2 image specified in that file and the .image file itself.
 	my $imagefile = $base . ".image";
 	if (! -e $imagefile) {
-	    $imagefile = rel_path(dirname($base), "cert.image");
+	    # was:
+	    # $imagefile = rel_path(dirname($base), "cert.image");
+	    $imagefile = File::Spec->catfile(dirname($base), "cert.image");
 	    if (! -e $imagefile) {
 		$imagefile = 0;
 	    }
@@ -1384,7 +1396,27 @@ sub deps_dfs {
     }
 }
 
-    
+# Depth-first search through the dependency map in order to propagate requirements (e.g. hons-only)
+# from books with that cert_param to books that include them.
+sub propagate_reqparam {
+    my ($target, $paramname, $visited, $depmap) = @_;
+    if ($visited->{$target}) {
+	return;
+    }
+    $visited->{$target} = 1;
+    my $certdeps = cert_deps($target, $depmap);
+    my $set_param = 0;
+    foreach my $dep (@$certdeps) {
+	propagate_reqparam($dep, $paramname, $visited, $depmap);
+	$set_param = $set_param || cert_get_param($dep, $depmap, $paramname);
+    }
+    if ($set_param && ! cert_get_param($target, $depmap, $paramname)) {
+	my $params = cert_get_params($target, $depmap);
+	$params->{$paramname} = $set_param;
+    }
+}
+
+
 
 # During a dependency search, this is run with $target set to each
 # cert and source file in the dependencies of the top-level targets.
@@ -1698,7 +1730,7 @@ sub compute_savings
 sub store_cache {
     my ($cache, $fname) = @_;
     if ($fname) {
-	store([$cache_version_code, $cache], $fname);
+	nstore([$cache_version_code, $cache], $fname);
     }
 }
 
