@@ -256,10 +256,19 @@ the user wishes to parallelize the execution of the defined function.</p>")
        (def  (if already-definedp
                  nil
                `((defun ,exec-fn (,@list-args ,acc)
-                   (declare (xargs :guard ,(if (equal guard t)
-                                               `(true-listp ,acc)
-                                             `(and (true-listp ,acc)
-                                                   ,guard))
+                   (declare (xargs :guard
+                                   ;; Previously we required that acc was a
+                                   ;; true-listp in the guard.  But on reflection,
+                                   ;; this really isn't necessary, and omitting it
+                                   ;; can simplify the guards of other functions
+                                   ;; that are directly calling -exec functions.
+
+                                   ;; ,(if (equal guard t)
+                                   ;;      `(true-listp ,acc)
+                                   ;;    `(and (true-listp ,acc)
+                                   ;;          ,guard))
+                                   ,guard
+
                                    :mode ,mode
                                    ;; we tell ACL2 not to normalize because
                                    ;; otherwise type reasoning can rewrite the
@@ -311,12 +320,10 @@ the user wishes to parallelize the execution of the defined function.</p>")
                                     (cw "~|~%Optimizing definition of ~s0:~%  ~p1~%~%"
                                         ',list-fn ',ndef)))))
                         (defttag cutil-optimize)
+                        ;; To justify nreverse, exec-fn must never be memoized
+                        (never-memoize ,exec-fn)
                         (progn!
                          (set-raw-mode t)
-                         (when (boundp 'ACL2::*never-profile-ht*)
-                           ;; ACL2(h).  Never allow exec function to be
-                           ;; memoized, to justify nreverse
-                           (setf (gethash ',exec-fn ACL2::*never-profile-ht*) t))
                          ,ndef)
                         (defttag nil)))))
 
@@ -407,12 +414,12 @@ the user wishes to parallelize the execution of the defined function.</p>")
                               (,list-fn ,@(subst y x list-args)))))
 
           ,@(if nil-preservingp
-                `((defthm ,(mksym 'simpler-take-of- list-fn)
-                    (equal (simpler-take ,n (,list-fn ,@list-args))
-                           (,list-fn ,@(subst `(simpler-take ,n ,x) x list-args)))
+                `((defthm ,(mksym 'take-of- list-fn)
+                    (equal (take ,n (,list-fn ,@list-args))
+                           (,list-fn ,@(subst `(take ,n ,x) x list-args)))
                     :hints(("Goal"
-                            :in-theory (enable simpler-take)
-                            :induct (simpler-take ,n ,x)))))
+                            :in-theory (enable acl2::take-redefinition)
+                            :induct (take ,n ,x)))))
               nil)
 
           (defthm ,(mksym 'nthcdr-of- list-fn)
@@ -450,9 +457,9 @@ the user wishes to parallelize the execution of the defined function.</p>")
           ,@(if already-definedp
                 nil
               `((defthm ,(mksym exec-fn '-removal)
-                  (implies (force (true-listp ,acc))
-                           (equal (,exec-fn ,@list-args ,acc)
-                                  (revappend (,list-fn ,@list-args) ,acc)))
+                  ;; we don't need the hyp... (implies (force (true-listp ,acc))
+                  (equal (,exec-fn ,@list-args ,acc)
+                         (revappend (,list-fn ,@list-args) ,acc))
                   :hints(("Goal" :in-theory (enable ,exec-fn))))
 
                 ,@(if verify-guards
