@@ -27,12 +27,12 @@
 
 
 # make_cert_help.pl -- this is a companion file that is used by "make_cert".
-# It is the core script responsible for certifying a single ACL2 book.
-
-# The code here is largely similar to the %.cert: %.lisp rule from
+# It is the core script responsible for certifying a single ACL2 book.  The
+# code here is largely similar to the %.cert: %.lisp rule from
 # Makefile-generic, but with several extensions.  For instance,
 #
-#   - We try to gracefully NFS lag, and
+#   - We try to gracefully NFS lag
+#   - We support cert-flags comments that say how to run certify-book
 #   - We can certify certain books with other save-images, using .image files
 #   - We support .acl2x files and two-pass certification,
 #   - We support adding #PBS directives to limit memory and wall time
@@ -40,8 +40,8 @@
 #
 # We only expect to invoke this through make_cert, so it is not especially
 # user-friendly.  We rely on several environment variables that are given to us
-# by make_cert.  See make_cert for the defaults and meanings of ACL2,
-# COMPILE_FLG, and other variables used in this script.
+# by make_cert.  See make_cert for the defaults and meanings of, e.g., ACL2 and
+# other variables used in this script.
 
 # Usage: make_cert_help.pl TARGET STEP
 #   - TARGET is like "foo" for "foo.lisp"
@@ -58,10 +58,14 @@
 use warnings;
 use strict;
 use File::Spec;
+# problematic to get this from cpan on msys
+# use File::Which qw(which);
 use FindBin qw($RealBin);
 use POSIX qw(strftime);
 use Cwd;
 
+(do "$RealBin/paths.pl") or die ("Error loading $RealBin/paths.pl:\n!: $!\n\@: $@\n");
+
 sub trim
 {
 	my $str = shift;
@@ -120,7 +124,7 @@ sub find_matching_parens {
     }
 }
 
-   
+
 sub skip_certify_books {
     my $str = shift;
     my $pos = 0;
@@ -275,9 +279,11 @@ sub scan_for_set_max_mem
 	if ($line =~ m/^[^;]*\((acl2::)?set-max-mem (.*)\)/)
 	{
 	    my $gb = parse_max_mem_arg($2);
+	    close $fd;
 	    return $gb;
 	}
     }
+    close $fd;
 
     return 0;
 }
@@ -293,32 +299,32 @@ sub scan_for_set_max_time
 	if ($line =~ m/^[^;]*\((acl2::)?set-max-time ([0-9]*)\)/)
 	{
 	    my $minutes = $2;
+	    close $fd;
 	    return $minutes;
 	}
     }
+    close $fd;
     return 0;
 }
 
-sub collect_ls_dirs {
-# Hi.  This is a horrible hack to compensate for NFS lag.  In our
-# setup, it seems to help to run "ls" on the directories containing
-# our dependencies (otherwise they often are not there yet when we try
-# to load them.)  To minimize the impact of this atrocity, we collect
-# the full set of these directories and uniqify them so that our
-# remote job doesn't do lots of redundant work.
-    my ($prereqs, $startdir) = @_;
-    my %dirs = ();
+sub parse_certify_flags
+{
+    my $filename = shift;  # just for error messages
+    my $str = shift;       # contents of the .acl2 file as a string
+    my @lines = split /^/, $str;
 
-    foreach my $prereq (@$prereqs) {
-	(my $vol, my $dir, my $file) = File::Spec->splitpath("$startdir/$prereq");
-	my $fulldir = File::Spec->canonpath(File::Spec->catpath($vol, $dir, ""));
-
-	$dirs{$fulldir} = 1;
+    # default: any number of portcullis commands are fine, and let's set the
+    # compile flag to true.
+    my $ret = "? t";
+    my $seen = 0;
+    foreach my $line (@lines) {
+	chomp($line);
+	if ($line =~ m/^[ \t]*;[; \t]*cert-flags: (.*)$/) {
+	    die("Multiple cert-flags directives are not allowed.") if $seen;
+	    $ret = $1;
+	}
     }
-
-    my @dirnames = keys %dirs;
-
-    return \@dirnames;
+    return $ret;
 }
 
 my $TARGET = shift;
@@ -332,7 +338,6 @@ my $PREREQS = \@ARGV;
 # }
 
 my $startdir = getcwd;
-# my $prereq_dirs = collect_ls_dirs($PREREQS, $startdir);
 
 my $TARGETEXT;
 if ($STEP eq "complete" || $STEP eq "certify") {
@@ -350,13 +355,12 @@ if ($STEP eq "complete" || $STEP eq "certify") {
 # normalize acl2x flag to Boolean
 $ACL2X = ($ACL2X eq "yes") ? ":acl2x t" : "";
 
-my $INHIBIT     = $ENV{"INHIBIT"} || "";
-my $HEADER      = $ENV{"OUTFILE_HEADER"} || "";
-my $MAX_NFS_LAG = $ENV{"MAX_NFS_LAG"} || 100;
-my $DEBUG       = $ENV{"ACL2_BOOKS_DEBUG"} ? 1 : 0;
-my $FLAGS       = $ENV{"COMPILE_FLG"};
-my $TIME_CERT   = $ENV{"TIME_CERT"} ? 1 : 0;
-my $STARTJOB    = $ENV{"STARTJOB"} || "";
+my $INHIBIT        = $ENV{"INHIBIT"} || "";
+my $HEADER         = $ENV{"OUTFILE_HEADER"} || "";
+my $MAX_NFS_LAG    = $ENV{"MAX_NFS_LAG"} || 100;
+my $DEBUG          = $ENV{"ACL2_BOOKS_DEBUG"} ? 1 : 0;
+my $TIME_CERT      = $ENV{"TIME_CERT"} ? 1 : 0;
+my $STARTJOB       = $ENV{"STARTJOB"} || "";
 my $ON_FAILURE_CMD = $ENV{"ON_FAILURE_CMD"} || "";
 my $ACL2           = $ENV{"ACL2"} || "acl2";
 # Figure out what ACL2 points to before we switch directories.
@@ -367,12 +371,15 @@ if ($ENV{"ACL2_BIN_DIR"}) {
     print "canonical bin dir: $canon_bin_dir\n" if $DEBUG;
     $ENV{"PATH"} = $ENV{"PATH"} ? "$canon_bin_dir:$ENV{'PATH'}" : $canon_bin_dir;
 }
-
-my $default_acl2 = `which $ACL2 2>/dev/null`;
+# fix up the acl2 path as in cert.pl
+my $devnull = File::Spec->devnull;
+$ACL2 = path_import($ACL2);
+my $default_acl2 = `which $ACL2 2>$devnull`;
 if (($? >> 8) != 0) {
     print "Error: failed to find \$ACL2 ($ACL2) in the PATH.\n";
     exit(1);
 }
+chomp($default_acl2);
 
 my $PCERT = "";
 if ($STEP eq "pcertify") {
@@ -394,7 +401,6 @@ if ($DEBUG)
     print "-- TARGETEXT    = $TARGETEXT\n";
     print "-- INHIBIT      = $INHIBIT\n";
     print "-- MAX_NFS_LAG  = $MAX_NFS_LAG\n";
-    print "-- FLAGS        = $FLAGS\n";
     print "-- PCERT        = $PCERT\n";
     print "-- ACL2X        = $ACL2X\n";
     print "-- HEADER       = $HEADER\n";
@@ -404,8 +410,9 @@ if ($DEBUG)
 my $full_file = File::Spec->rel2abs($TARGET);
 (my $vol, my $dir, my $file) = File::Spec->splitpath($full_file);
 my $goal = "$file.$TARGETEXT";
+my $printgoal = path_export("$full_file.$TARGETEXT");
 
-print "Making $dir$goal on " . strftime('%d-%b-%Y %H:%M:%S',localtime) . "\n";
+print "Making $printgoal on " . strftime('%d-%b-%Y %H:%M:%S',localtime) . "\n";
 
 my $fulldir = File::Spec->canonpath(File::Spec->catpath($vol, $dir, ""));
 print "-- Entering directory $fulldir\n" if $DEBUG;
@@ -423,82 +430,68 @@ remove_file_if_exists($outfile);
 
 write_whole_file($outfile, $HEADER);
 
-## Consider just using "mv"/"touch" for the complete step.
-if ($STEP eq "complete") {
-    ## BOZO this is horrible and only works for CCL/linux and probably the file time thing
-    ## is also wrong.
-    my $cmd = "(time ((cp $file.pcert1 $file.cert; if [[ $file.lx64fsl -nt $file.pcert1 ]]; then touch $file.lx64fsl; fi) >> $outfile)) 2> $timefile";
-    if (system($cmd) != 0) {
-	die("Failed to move $file.pcert1 to $file.cert\n");
-    }
-    $status = 43;
-} else {
+
 
 # Override ACL2 per the image file, as appropriate.
-    my $acl2 = read_whole_file_if_exists("$file.image");
-    $acl2 = read_whole_file_if_exists("cert.image") if !$acl2;
-    $acl2 = $default_acl2 if !$acl2;
-    $acl2 = trim($acl2);
-    $ENV{"ACL2"} = $acl2;
-    print "-- Image to use = $acl2\n" if $DEBUG;
-    die("Can't determine which ACL2 to use.") if !$acl2;
+my $acl2 = read_whole_file_if_exists("$file.image");
+$acl2 = read_whole_file_if_exists("cert.image") if !$acl2;
+$acl2 = $default_acl2 if !$acl2;
+$acl2 = trim($acl2);
+$ENV{"ACL2"} = $acl2;
+print "-- Image to use = $acl2\n" if $DEBUG;
+die("Can't determine which ACL2 to use.") if !$acl2;
 
 # ------------ TEMPORARY LISP FILE FOR ACL2 INSTRUCTIONS ----------------------
 
-    my $rnd = int(rand(2**30));
-    my $tmpbase = "workxxx.$goal.$rnd";
+my $rnd = int(rand(2**30));
+my $tmpbase = "workxxx.$goal.$rnd";
 # upper-case .LISP so if it doesn't get deleted, we won't try to certify it
-    my $lisptmp = "$tmpbase.LISP";
-    print "-- Temporary lisp file: $lisptmp\n" if $DEBUG;
+my $lisptmp = "$tmpbase.LISP";
+print "-- Temporary lisp file: $lisptmp\n" if $DEBUG;
 
-    my $instrs = "";
+my $instrs = "";
 
 # I think this strange :q/lp dance is needed for lispworks or something?
-    $instrs .= "(acl2::value :q)\n";
-    $instrs .= "(in-package \"ACL2\")\n";
-    $instrs .= "#+acl2-hons (profile-fn 'prove)\n";
-    $instrs .= "#+acl2-hons (profile-fn 'certify-book-fn)\n";
-    $instrs .= "(acl2::lp)\n\n";
+$instrs .= "(acl2::value :q)\n";
+$instrs .= "(in-package \"ACL2\")\n";
+$instrs .= "#+acl2-hons (profile-fn 'prove)\n";
+$instrs .= "#+acl2-hons (profile-fn 'certify-book-fn)\n";
+$instrs .= "(acl2::lp)\n\n";
 #    $instrs .= "(set-debugger-enable :bt)\n";
-    $instrs .= "(set-write-acl2x t state)\n" if ($STEP eq "acl2x");
-    $instrs .= "(set-write-acl2x '(t) state)\n" if ($STEP eq "acl2xskip");
-    $instrs .= "$INHIBIT\n" if ($INHIBIT);
+$instrs .= "(set-write-acl2x t state)\n" if ($STEP eq "acl2x");
+$instrs .= "(set-write-acl2x '(t) state)\n" if ($STEP eq "acl2xskip");
+$instrs .= "$INHIBIT\n" if ($INHIBIT);
+$instrs .= "\n";
 
-    $instrs .= "\n";
-
-
-    my $cert_cmd = "#!ACL2 (er-progn (time\$ (certify-book \"$file\" ? $FLAGS $PCERT $ACL2X))
-                                 (value (prog2\$ #+acl2-hons (memsum)
-                                                 #-acl2-hons nil
-                                                 (exit 43))))";
 
 # Get the certification instructions from foo.acl2 or cert.acl2, if either
 # exists, or make a generic certify-book command.
-    if (-f "$file.acl2") {
-	$instrs .= "; instructions from $file.acl2\n";
-	$instrs .= "; (omitting any certify-book line):\n";    
-	$instrs .= read_file_except_certify("$file.acl2");
-	$instrs .= "\n; certify-book command added automatically:\n";
-	$instrs .= "$cert_cmd\n\n";
-    }
-    elsif (-f "cert.acl2") {
-	$instrs .= "; instructions from cert.acl2:\n";
-	$instrs .= read_whole_file("cert.acl2");
-	$instrs .= "\n; certify-book command added automatically:\n";
-	$instrs .= "$cert_cmd\n\n";
-    }
-    else {
-	$instrs .= "; certify-book generated automatically:\n";
-	$instrs .= "$cert_cmd\n\n";
-    }
+my $acl2file = (-f "$file.acl2") ? "$file.acl2"
+    : (-f "cert.acl2")  ? "cert.acl2"
+    : "";
 
-    if ($DEBUG) {
-	print "-- ACL2 Instructions: $lisptmp --\n";
-	print "$instrs\n";
-	print "-------------------------------------------------------\n\n";
-    }
+my $usercmds = $acl2file ? read_file_except_certify($acl2file) : "";
 
-    write_whole_file($lisptmp, $instrs);
+$instrs .= "; instructions from .acl2 file $acl2file:\n";
+$instrs .= "$usercmds\n";
+
+my $cert_flags = parse_certify_flags($acl2file, $usercmds);
+$instrs .= "; certify-book command flags: $cert_flags\n";
+
+my $cert_cmd = "#!ACL2 (er-progn (time\$ (certify-book \"$file\" $cert_flags $PCERT $ACL2X))
+                                 (value (prog2\$ #+acl2-hons (memsum)
+                                                 #-acl2-hons nil
+                                                 (exit 43))))\n";
+
+$instrs .= $cert_cmd;
+
+if ($DEBUG) {
+    print "-- ACL2 Instructions: $lisptmp --\n";
+    print "$instrs\n";
+    print "-------------------------------------------------------\n\n";
+}
+
+write_whole_file($lisptmp, $instrs);
 
 
 
@@ -509,9 +502,9 @@ if ($STEP eq "complete") {
     my $shinsts = "#!/bin/sh\n\n";
 
 # If we find a set-max-mem line, add 3 gigs of padding for the stacks and to
-# allow the lisp to have some room to go over.  Default to 6 gigs.
+# allow the lisp to have some room to go over.  Default to 4 gigs.
     my $max_mem = scan_for_set_max_mem("$file.lisp");
-    $max_mem = $max_mem ? ($max_mem + 3) : 6;
+    $max_mem = $max_mem ? ($max_mem + 3) : 4;
 
 # If we find a set-max-time line, honor it directly; otherwise default to
 # 240 minutes.
@@ -542,7 +535,7 @@ if ($STEP eq "complete") {
     $shinsts .= "echo Start of output: >> $outfile\n";
     $shinsts .= "echo >> $outfile\n";
 
-
+    $shinsts .= "echo ACL2_SYSTEM_BOOKS: \${ACL2_SYSTEM_BOOKS} >> $outfile\n";
 
     if ($TIME_CERT) {
 	$shinsts .= "(time (($acl2 < $lisptmp 2>&1) >> $outfile)) 2> $timefile\n";
@@ -573,7 +566,7 @@ if ($STEP eq "complete") {
     unlink($lisptmp) if !$DEBUG;
     unlink($shtmp) if !$DEBUG;
 
-}
+
 
 # Success or Failure Detection -------------------------------
 
@@ -597,7 +590,7 @@ if ($status == 43) {
 }
 
 if ($success) {
-    print "Successfully built $dir$goal\n";
+    print "Successfully built $printgoal\n";
 } else {
     my $taskname = ($STEP eq "acl2x" || $STEP eq "acl2xskip") ? "ACL2X GENERATION" :
 	($STEP eq "certify")  ? "CERTIFICATION" :
