@@ -1,5 +1,14 @@
 (in-package "ACL2")
 
+#||
+
+; [Jared and Sol]: fool make_cert_help.pl into allowing more memory for this
+; book. We would just include centaur/misc/memory-mgmt, but that has a ttag.
+
+(set-max-mem (* 6 (expt 2 30)))
+
+||#
+
 (include-book "x86-state")
 
 (local
@@ -495,6 +504,10 @@
            (member a y)
            (member a z))))
 
+;; Jared added this after fixing up std/lists/sets stuff
+(local (in-theory (e/d (revappend)
+                       (revappend-removal))))
+
 (defthm no-duplicatesp-sorted-revappend-2
   (implies (not (no-duplicatesp-sorted y))
            (not (no-duplicatesp-sorted (revappend x y)))))
@@ -1494,8 +1507,8 @@
                          (logior x2 y2))
                   (and (equal x1 x2)
                        (equal y1 y2))))
-  :hints (("Subgoal 1.1.1.1.1" :cases ((< k j)
-                                       (< j k))))
+  :hints (("Subgoal 1.1.1.1" :cases ((< k j)
+                                     (< j k))))
   :otf-flg t)
 
 (defthm logior-2
