@@ -28,10 +28,10 @@
 (include-book "char-case")
 (include-book "cat")
 (local (include-book "arithmetic"))
-(local (include-book "unicode/rev" :dir :system))
-(local (include-book "unicode/coerce" :dir :system))
-(local (include-book "unicode/take" :dir :system))
-(local (include-book "unicode/nthcdr" :dir :system))
+(local (include-book "std/lists/rev" :dir :system))
+(local (include-book "std/lists/coerce" :dir :system))
+(local (include-book "std/lists/take" :dir :system))
+(local (include-book "std/lists/nthcdr" :dir :system))
 (local (include-book "subseq"))
 
 (local (defthm append-singleton-crock
@@ -280,10 +280,8 @@ least we're better when no work needs to be done:</p>
            (if (not (string-has-some-down-alpha-p x 0 xl))
                ;; Avoid consing when no characters need to be converted.
                x
-             (let* ((acc     (upcase-string-aux x 0 xl nil))
-                    (rev-ans (coerce acc 'string)))
-               ;; Generally best to reverse the string, rather than the chars.
-               (reverse rev-ans))))))
+             (rchars-to-string
+              (upcase-string-aux x 0 xl nil))))))
 
   (defthm string-has-some-down-alpha-p-redef
     (implies (and (stringp x)
@@ -385,10 +383,7 @@ make this fast.</p>"
            (if (not (string-has-some-up-alpha-p x 0 xl))
                ;; Avoid consing when no characters need to be converted.
                x
-             (let* ((acc     (downcase-string-aux x 0 xl nil))
-                    (rev-ans (coerce acc 'string)))
-               ;; Generally best to reverse the string, rather than the chars.
-               (reverse rev-ans))))))
+             (rchars-to-string (downcase-string-aux x 0 xl nil))))))
 
   (defthm string-has-some-up-alpha-p-redef
     (implies (and (stringp x)
