@@ -208,24 +208,37 @@ do this:</p>
      :skip t))
 })
 
-<p>There is an older, alternate syntax for @('make-flag') that is still
-available.  To encourage transitioning to the new syntax, the old syntax is not
-documented and should usually not be used.  However, the old syntax is very
-convenient for skipping theorems, because it is more concise and doesn't
-require theorem names for skipped theorems.  Here's an example:</p>
-
+<p>Sometimes the theorem you want is inductive in such a way that some
+functions are irrelevant; nothing needs to be proved about them in order to
+prove the desired theorem about the others.  The :skip keyword can be used with
+a theorem of T to do this:</p>
 @({
  (defthm-pseudo-termp
-   (list
-    (booleanp (pseudo-term-listp acl2::lst))
-    :skip t)
    (defthm type-of-pseudo-termp
-     (booleanp (pseudo-termp acl2::x))
+     (booleanp (pseudo-termp x))
      :rule-classes :type-prescription
-     :flag term))
+     :flag term)
+   (defthm type-of-pseudo-term-listp
+     t
+     :flag list
+     :skip t))
+})
+<p>Alternatively, you can provide the :skip-others keyword to the top-level
+macro and simply leave out the trivial parts:</p>
+@({
+ (defthm-pseudo-termp
+   (defthm type-of-pseudo-termp
+     (booleanp (pseudo-termp x))
+     :rule-classes :type-prescription
+     :flag term)
+   :skip-others t)
 })
 
 
+<p>There is an older, alternate syntax for @('make-flag') that is still
+available.  To encourage transitioning to the new syntax, the old syntax is not
+documented and should usually not be used.</p>
+
 <h3>Advanced Hints</h3>
 
 <p>For advanced users, note that each individual \"theorem\" can have its own
@@ -475,13 +488,15 @@ effect.  We simply translate subgoal hints to computed hints:</p>
                (list `(otherwise ,new-body)))))
     (er hard 'make-flag-body-aux "Never get here.")))
 
-(defun make-flag-body (fn-name flag-var alist hints world)
+(defun make-flag-body (fn-name flag-var alist hints ruler-extenders world)
   (let ((formals (merge-formals alist world)))
   `(defun-nx ,fn-name (,flag-var . ,formals)
      (declare (xargs :verify-guards nil
                      :normalize nil
                      :measure ,(make-flag-measure flag-var alist world)
                      :hints ,hints
+                     ,@(and ruler-extenders
+                            `(:ruler-extenders ,ruler-extenders))
                      :well-founded-relation ,(get-wfr (caar alist)
                                                       world)
                      :mode :logic)
@@ -598,7 +613,7 @@ effect.  We simply translate subgoal hints to computed hints:</p>
 
 
 
-(defun pair-up-cases-with-thmparts (alist thmparts)
+(defun pair-up-cases-with-thmparts (alist thmparts skip-ok)
   ;; Each thmpart is an thing like
   ;; _either_ (flag <thm-body> :name ... :rule-classes ... :doc ...)
   ;;;    (for backwards compatibility)
@@ -607,23 +622,24 @@ effect.  We simply translate subgoal hints to computed hints:</p>
   (if (consp alist)
       (let* ((flag   (cdar alist))
              (lookup (assoc-flag-in-thmparts flag thmparts)))
-        (if (not lookup)
+        (if (and (not lookup) (not skip-ok))
             (er hard 'pair-up-cases-with-thmparts
                 "Expected there to be a case for the flag ~s0.~%" flag)
-          (let ((body (if (eq (car lookup) 'defthm)
-                          ;; (defthm name body ...)
-                          (caddr lookup)
-                        ;; (flag body ...)
-                        (cadr lookup))))
+          (let ((body (cond ((not lookup) t)
+                            ((eq (car lookup) 'defthm)
+                             ;; (defthm name body ...)
+                             (caddr lookup))
+                            (t ;; (flag body ...)
+                             (cadr lookup)))))
             (if (consp (cdr alist))
                 (cons `(,flag ,body)
-                      (pair-up-cases-with-thmparts (cdr alist) thmparts))
+                      (pair-up-cases-with-thmparts (cdr alist) thmparts skip-ok))
               (list `(otherwise ,body))))))
     (er hard 'pair-up-cases-with-thmparts
         "Never get here.")))
 
 
-(defun pair-up-cases-with-hints (alist thmparts)
+(defun pair-up-cases-with-hints (alist thmparts skip-ok)
   ;; Each thmpart is an thing like
   ;; _either_ (flag <thm-body> :name ... :rule-classes ... :doc ...)
   ;;;    (for backwards compatibility)
@@ -633,11 +649,14 @@ effect.  We simply translate subgoal hints to computed hints:</p>
       (let* ((flag   (cdar alist))
              (lookup (assoc-flag-in-thmparts flag thmparts)))
         (if (not lookup)
-            (er hard 'pair-up-cases-with-hints
-                "Expected there to be a case for the flag ~s0.~%" flag)
+            (if skip-ok
+                (cons (cons flag nil)
+                      (pair-up-cases-with-hints (cdr alist) thmparts skip-ok))
+              (er hard 'pair-up-cases-with-hints
+                  "Expected there to be a case for the flag ~s0.~%" flag))
           (let ((hints (extract-keyword-from-args :hints lookup)))
             (cons (cons flag hints)
-                  (pair-up-cases-with-hints (cdr alist) thmparts)))))
+                  (pair-up-cases-with-hints (cdr alist) thmparts skip-ok)))))
     nil))
 
 (defun flag-thm-entry-thmname (explicit-name flag entry)
@@ -663,7 +682,8 @@ given.  The following theorem does not have a name: ~x0~%" entry)))))
   (if (consp alist)
       (let* ((flag (cdar alist))
              (lookup (assoc-flag-in-thmparts flag thmparts)))
-        (if (extract-keyword-from-args :skip (cddr lookup))
+        (if (or (not lookup)
+                (extract-keyword-from-args :skip (cddr lookup)))
             (make-defthm-macro-fn-aux
              lemma-name explicit-name flag-var (cdr alist) thmparts)
           ;; Not checking for lookup, already did it when we did cases.
@@ -686,6 +706,16 @@ given.  The following theorem does not have a name: ~x0~%" entry)))))
                    lemma-name explicit-name flag-var (cdr alist) thmparts)))))
     nil))
 
+(defun find-first-thm-name (thmparts)
+  (if (atom thmparts)
+      (er hard? 'find-first-thm-name
+          "No explicit name given, and no theorems are given names?")
+    (if (extract-keyword-from-args :skip (cddr (car thmparts)))
+        (find-first-thm-name (cdr thmparts))
+      (flag-thm-entry-thmname
+       nil (flag-from-thmpart (car thmparts)) (car thmparts)))))
+       
+
 
 
 (defun make-defthm-macro-fn (args alist flag-var flag-fncall)
@@ -699,10 +729,12 @@ given.  The following theorem does not have a name: ~x0~%" entry)))))
                     explicit-name)
                  (intern-in-package-of-symbol
                   (concatenate 'string "FLAG-LEMMA-FOR-"
-                               (symbol-name (car flag-fncall)))
+                               (symbol-name
+                                (find-first-thm-name thmparts)))
                   (car flag-fncall))))
          (instructions (extract-keyword-from-args :instructions args))
          (user-hints (extract-keyword-from-args :hints args))
+         (skip-ok (extract-keyword-from-args :skip-others args))
          (hints (and (not instructions)
                      (append
                       (if (and (consp (car user-hints))
@@ -724,13 +756,14 @@ given.  The following theorem does not have a name: ~x0~%" entry)))))
                       (list
                        `(flag-hint-cases
                          ,flag-var
-                         . ,(pair-up-cases-with-hints alist thmparts)))))))
+                         . ,(pair-up-cases-with-hints alist thmparts skip-ok)))))))
 
     `(progn
        (encapsulate
         ()
         (local (defthm ,name
-                 (case ,flag-var . ,(pair-up-cases-with-thmparts alist thmparts))
+                 (case ,flag-var . ,(pair-up-cases-with-thmparts
+                                     alist thmparts skip-ok))
                  :rule-classes nil
                  :hints ,hints
                  :instructions ,instructions
@@ -816,7 +849,7 @@ given.  The following theorem does not have a name: ~x0~%" entry)))))
           (flag-table-events (cdr alist) entry))))
 
 (defun make-flag-fn (flag-fn-name clique-member-name flag-var flag-mapping hints
-                                  defthm-macro-name local world)
+                                  defthm-macro-name local ruler-extenders world)
   (let* ((flag-var (or flag-var
                        (intern-in-package-of-symbol "FLAG" flag-fn-name)))
          (alist (or flag-mapping
@@ -836,7 +869,7 @@ given.  The following theorem does not have a name: ~x0~%" entry)))))
       ,@(and (not local) '((set-ignore-ok t))) ;; can't wrap this in local --- fubar!
 
       (,(if local 'local 'id)
-       ,(make-flag-body flag-fn-name flag-var alist hints world))
+       ,(make-flag-body flag-fn-name flag-var alist hints ruler-extenders world))
       ,(make-defthm-macro defthm-macro-name alist flag-var
                           `(,flag-fn-name ,flag-var . ,formals))
 
@@ -882,7 +915,8 @@ given.  The following theorem does not have a name: ~x0~%" entry)))))
                      flag-mapping
                      hints
                      defthm-macro-name
-                     local)
+                     local
+                     ruler-extenders)
   `(make-event (make-flag-fn ',flag-fn-name
                              ',clique-member-name
                              ',flag-var
@@ -890,6 +924,7 @@ given.  The following theorem does not have a name: ~x0~%" entry)))))
                              ',hints
                              ',defthm-macro-name
                              ',local
+                             ',ruler-extenders
                              (w state))))
 
 
@@ -949,6 +984,16 @@ given.  The following theorem does not have a name: ~x0~%" entry)))))
                   :doc nil)
             (list (booleanp (pseudo-term-listp lst))))))
 
+  (encapsulate
+   nil
+   (local (defthm-pseudo-termp type-of-pseudo-termp2
+            (defthm booleanp-of-pseudo-termp
+              (booleanp (pseudo-termp x))
+              :rule-classes :rewrite
+              :doc nil
+              :flag term)
+            :skip-others t)))
+
 
   (encapsulate
    nil
