@@ -17,13 +17,15 @@
 
 (in-package "ACL2")
 (include-book "utf8-decode")
-(include-book "take-bytes")
+(include-book "std/io/take-bytes" :dir :system)
 ;; (local (include-book "open-input-channel"))
 ;; (local (include-book "close-input-channel"))
-(local (include-book "system/io" :dir :system))
+(local (include-book "std/io/base" :dir :system))
 (local (include-book "tools/mv-nth" :dir :system))
 (set-state-ok t)
 
+(local (in-theory (disable take-redefinition)))
+
 
 ;; We now want to recreate our utf8=>ustring function but directly using the
 ;; file reading operations.  We begin by writing equivalents of "take" and
@@ -210,7 +212,7 @@
 ;; The proof is sort of cute.  We rewrite everything to be in terms of
 ;; read-byte$-all.  For example, we first show that read-byte$ is nothing more
 ;; than the car of read-byte$-all.  Similarly, in take-bytes.lisp, we have
-;; shown that take-bytes is just the simpler-take of read-byte$-all.
+;; shown that take-bytes is just the take of read-byte$-all.
 
 (local (defthm car-of-read-byte$
          (implies (and (force (state-p state))
@@ -256,7 +258,7 @@
                                       acc)))
   :hints(("Goal"
           :in-theory (e/d (read-utf8-fast utf8=>ustring-fast
-                                          simpler-take)
+                                          take-redefinition)
                           (nthcdr-bytes-2
                            nthcdr-bytes-3
                            nthcdr-bytes-4))
@@ -273,7 +275,7 @@
 
 (encapsulate
  ()
- (local (in-theory (enable simpler-take)))
+ (local (in-theory (enable take-redefinition)))
 
  (local (defthm terrible-lemma-1
           (implies (and (integerp x)
@@ -423,7 +425,7 @@
                   :use ((:instance uchar?-of-utf8-combine4
                                    (x1 244)))))))
 
- (local (include-book "signed-byte-listp"))
+ (local (include-book "std/io/signed-byte-listp" :dir :system))
 
  (local (defthm unsigned-byte-listp-8-of-car-of-read-byte$-all-forward
           (implies (and (force (state-p1 state))
@@ -595,6 +597,8 @@
            (state-p1 (mv-nth 1 (read-utf8 filename state))))
   :hints(("Goal" :in-theory (enable read-utf8))))
 
+#|| No longer true, given change to read-file-bytes in
+    std/io/read-file-bytes.lisp, svn 1477.
 (defthm car-of-read-utf8-when-file-cannot-be-opened
   (implies (and (stringp (mv-nth 0 (read-file-bytes filename state)))
                 (force (state-p1 state))
@@ -602,6 +606,7 @@
            (equal (mv-nth 0 (read-utf8 filename state))
                   (mv-nth 0 (read-file-bytes filename state))))
   :hints(("Goal" :in-theory (enable read-file-bytes read-utf8))))
+||#
 
 (defthm car-of-read-utf8-when-file-can-be-opened
   (implies (and (not (stringp (mv-nth 0 (read-file-bytes filename state))))
