@@ -153,17 +153,17 @@ and \"preformatted,\" i.e., newlines and spaces should be preserved.</li>
 
 </ul>
 
-<p><b>However</b>, it's often better to use the preprocessor's @('@@('...')')
-and @('@@({...})') macros.  These are nice because they automatically escape
-special HTML characters like &lt; into &amp;lt;, and also automatically add
-hyperlinks to documented functions.</p>
+<p><b>However</b>, it's often better to use the preprocessor's
+<tt>@@('...')</tt> and <tt>@@({...})</tt> macros.  These are nice because they
+automatically escape special HTML characters like &lt; into &amp;lt;, and also
+automatically add hyperlinks to documented functions.</p>
 
-<p>If you do decide to write raw @('&lt;code&gt;') blocks, your lisp forms
-should usually be <b>indented one space</b> to prevent Emacs problems.  For
+<p>If you do decide to write raw @('<code>') blocks, your lisp forms should
+usually be <b>indented one space</b> to prevent Emacs problems.  For
 instance:</p>
 
 @({
- (defdoc foo
+ (defxdoc foo
    :long \"<h3>How to format @('<code>') blocks</h3>
 
  <p>GOOD -- the form is indented one space:</p>
@@ -180,7 +180,7 @@ instance:</p>
 })
 
 <p>Without this leading space, Emacs can become confused and think that
-@('(my-lisp-form ...)'), rather than @('(defdoc foo ...)'), is the top-level
+@('(my-lisp-form ...)'), rather than @('(defxdoc foo ...)'), is the top-level
 expression.  This can ruin syntax highlighting and also commands like @('C-t
 e') for sending s-expressions to the @('*shell*') window.</p>
 
@@ -441,17 +441,31 @@ to view @('preview.html') without any additional steps.</p>
 <h3>HTML and Other Formats</h3>
 
 <p>You can generate a plain HTML or TEXT version of your manual by using
-@('make html') or @('make text').  We might add support for TEXINFO or other
-formats in the future.</p>
+@('make html') or @('make text') from within the directory target of the above
+mentioned @('xdoc::save') command (in this example, the @('mylib-manual')
+directory).  We might add support for TEXINFO or other formats in the
+future.</p>
 
 <p>After running @('make html'), you may wish to open @('frames2.html') and
 @('frames3.html'), which allow you to navigate the HTML manual much like
-@('preview.html') allows you to navigate the XML version.</p>
+@('preview.html') allows you to navigate the XML version.  These pages accept
+an optional argument named <tt>topic</tt> that tells the browser to
+automatically go to a particular topic.  For example, one can go to the
+<tt>XDOC::save</tt> topic by using the url
+<tt>frames3.html?topic=XDOC____SAVE.html</tt>.</p>
 
 <p>Converting to HTML is a good idea because it ensures that all of your tags
 are balanced on every page.  Without this sanity check, your manual might
 contain errors that will prevent some topics from being loaded in a web
-browser.</p>")
+browser.</p>
+
+<p>Creating the HTML code requires Xalan-C++.  Xalan is distributed with many
+Linux distributions.  For example, on Ubuntu, one can run <tt>sudo apt-get
+install xalan</tt> to install it.  Alternatively, see <a
+href=\"http://xml.apache.org/xalan-c/\">Apache Xalan</a> to download.  We have
+accomodated the various versions of Xalan that we know about and use, but we
+welcome modifications to file <tt>support/Makefile-trans</tt> if you wish to
+use a version we do not currently support.</p>")
 
 
 (defxdoc emacs-links
@@ -807,4 +821,4 @@ not introduce a new scope, whereas a @('defsection') does.</p>")
 <li>@('&nbsp;')  becomes &nbsp; (this one can be hard to see)</li>
 <li>@('&mdash;') becomes &mdash;</li>
 <li>@('&rarr;')  becomes &rarr;</li>
-</ul>")
\ No newline at end of file
+</ul>")
